@@ -47,16 +47,6 @@ public class HeapRowVector extends AbstractStructVector
         return vectorizedColumnBatch;
     }
 
-    @Override
-    public void reset() {
-        super.reset();
-        for (ColumnVector field : children) {
-            if (field instanceof WritableColumnVector) {
-                ((WritableColumnVector) field).reset();
-            }
-        }
-    }
-
     @Override
     void reserveForHeapVector(int newCapacity) {
         // Nothing to store.
