@@ -44,6 +44,15 @@ public class RoaringBitmap32 {
         this.roaringBitmap = roaringBitmap;
     }
 
+    /**
+     * Note: the result is read only, do not call any modify operation outside.
+     *
+     * @return the roaringBitmap
+     */
+    protected RoaringBitmap get() {
+        return roaringBitmap;
+    }
+
     public void add(int x) {
         roaringBitmap.add(x);
     }
