@@ -23,11 +23,11 @@ import org.apache.paimon.deletionvectors.DeletionVectorsMaintainer;
 import org.apache.paimon.format.FileFormatDiscover;
 import org.apache.paimon.fs.FileIO;
 import org.apache.paimon.manifest.ManifestCacheFilter;
+import org.apache.paimon.operation.AppendFileStoreWrite;
 import org.apache.paimon.operation.AppendOnlyFileStoreScan;
-import org.apache.paimon.operation.AppendOnlyFileStoreWrite;
-import org.apache.paimon.operation.AppendOnlyFixedBucketFileStoreWrite;
-import org.apache.paimon.operation.AppendOnlyUnawareBucketFileStoreWrite;
+import org.apache.paimon.operation.BaseAppendFileStoreWrite;
 import org.apache.paimon.operation.BucketSelectConverter;
+import org.apache.paimon.operation.BucketedAppendFileStoreWrite;
 import org.apache.paimon.operation.RawFileSplitRead;
 import org.apache.paimon.predicate.Predicate;
 import org.apache.paimon.schema.SchemaManager;
@@ -36,6 +36,8 @@ import org.apache.paimon.table.BucketMode;
 import org.apache.paimon.table.CatalogEnvironment;
 import org.apache.paimon.types.RowType;
 
+import javax.annotation.Nullable;
+
 import java.util.Comparator;
 import java.util.List;
 import java.util.Optional;
@@ -88,19 +90,21 @@ public class AppendOnlyFileStore extends AbstractFileStore<InternalRow> {
     }
 
     @Override
-    public AppendOnlyFileStoreWrite newWrite(String commitUser) {
-        return newWrite(commitUser, null);
+    public BaseAppendFileStoreWrite newWrite(String commitUser) {
+        return newWrite(commitUser, null, null);
     }
 
     @Override
-    public AppendOnlyFileStoreWrite newWrite(
-            String commitUser, ManifestCacheFilter manifestFilter) {
+    public BaseAppendFileStoreWrite newWrite(
+            String commitUser,
+            @Nullable ManifestCacheFilter manifestFilter,
+            @Nullable Integer writeId) {
         DeletionVectorsMaintainer.Factory dvMaintainerFactory =
                 options.deletionVectorsEnabled()
                         ? DeletionVectorsMaintainer.factory(newIndexFileHandler())
                         : null;
         if (bucketMode() == BucketMode.BUCKET_UNAWARE) {
-            return new AppendOnlyUnawareBucketFileStoreWrite(
+            return new AppendFileStoreWrite(
                     fileIO,
                     newRead(),
                     schema.id(),
@@ -113,7 +117,7 @@ public class AppendOnlyFileStore extends AbstractFileStore<InternalRow> {
                     dvMaintainerFactory,
                     tableName);
         } else {
-            return new AppendOnlyFixedBucketFileStoreWrite(
+            return new BucketedAppendFileStoreWrite(
                     fileIO,
                     newRead(),
                     schema.id(),
