@@ -20,9 +20,7 @@ package org.apache.paimon.catalog;
 
 import org.apache.paimon.Snapshot;
 import org.apache.paimon.partition.PartitionStatistics;
-import org.apache.paimon.utils.SnapshotManager;
 
-import java.io.Serializable;
 import java.util.List;
 
 /** Interface to commit snapshot atomically. */
@@ -30,9 +28,4 @@ public interface SnapshotCommit extends AutoCloseable {
 
     boolean commit(Snapshot snapshot, String branch, List<PartitionStatistics> statistics)
             throws Exception;
-
-    /** Factory to create {@link SnapshotCommit}. */
-    interface Factory extends Serializable {
-        SnapshotCommit create(Identifier identifier, SnapshotManager snapshotManager);
-    }
 }
