@@ -18,6 +18,7 @@
 
 package org.apache.paimon.index;
 
+import org.apache.paimon.Snapshot;
 import org.apache.paimon.data.BinaryRow;
 
 import javax.annotation.Nullable;
@@ -34,6 +35,6 @@ public interface IndexMaintainer<T> {
     /** Factory to restore {@link IndexMaintainer}. */
     interface Factory<T> {
         IndexMaintainer<T> createOrRestore(
-                @Nullable Long snapshotId, BinaryRow partition, int bucket);
+                @Nullable Snapshot snapshot, BinaryRow partition, int bucket);
     }
 }
