@@ -78,9 +78,12 @@ public abstract class AbstractFileStoreScan implements FileStoreScan {
     private final TableSchema schema;
 
     private Snapshot specifiedSnapshot = null;
+    private boolean onlyReadRealBuckets = false;
+    private Integer specifiedBucket = null;
     private Filter<Integer> bucketFilter = null;
     private BiFilter<Integer, Integer> totalAwareBucketFilter = null;
     protected ScanMode scanMode = ScanMode.ALL;
+    private Integer specifiedLevel = null;
     private Filter<Integer> levelFilter = null;
     private Filter<ManifestEntry> manifestEntryFilter = null;
     private Filter<String> fileNameFilter = null;
@@ -118,15 +121,29 @@ public abstract class AbstractFileStoreScan implements FileStoreScan {
         return this;
     }
 
+    @Override
+    public FileStoreScan withPartitionsFilter(List<Map<String, String>> partitions) {
+        manifestsReader.withPartitionsFilter(partitions);
+        return this;
+    }
+
     @Override
     public FileStoreScan withPartitionFilter(PartitionPredicate predicate) {
         manifestsReader.withPartitionFilter(predicate);
         return this;
     }
 
+    @Override
+    public FileStoreScan onlyReadRealBuckets() {
+        manifestsReader.onlyReadRealBuckets();
+        this.onlyReadRealBuckets = true;
+        return this;
+    }
+
     @Override
     public FileStoreScan withBucket(int bucket) {
-        this.bucketFilter = i -> i == bucket;
+        manifestsReader.withBucket(bucket);
+        specifiedBucket = bucket;
         return this;
     }
 
@@ -175,6 +192,13 @@ public abstract class AbstractFileStoreScan implements FileStoreScan {
         return this;
     }
 
+    @Override
+    public FileStoreScan withLevel(int level) {
+        manifestsReader.withLevel(level);
+        this.specifiedLevel = level;
+        return this;
+    }
+
     @Override
     public FileStoreScan withLevelFilter(Filter<Integer> levelFilter) {
         this.levelFilter = levelFilter;
@@ -491,6 +515,14 @@ public abstract class AbstractFileStoreScan implements FileStoreScan {
             }
 
             int bucket = bucketGetter.apply(row);
+            if (onlyReadRealBuckets && bucket < 0) {
+                return false;
+            }
+
+            if (specifiedBucket != null && bucket != specifiedBucket) {
+                return false;
+            }
+
             if (bucketFilter != null && !bucketFilter.test(bucket)) {
                 return false;
             }
@@ -500,7 +532,12 @@ public abstract class AbstractFileStoreScan implements FileStoreScan {
                 return false;
             }
 
-            if (levelFilter != null && !levelFilter.test(levelGetter.apply(row))) {
+            int level = levelGetter.apply(row);
+            if (specifiedLevel != null && level != specifiedLevel) {
+                return false;
+            }
+
+            if (levelFilter != null && !levelFilter.test(level)) {
                 return false;
             }
 
