@@ -37,7 +37,6 @@ import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
 import java.util.Collection;
-import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
@@ -106,10 +105,6 @@ public class TagDeletion extends FileDeletionBase<Snapshot> {
         cleanUnusedManifests(taggedSnapshot, skippingSet, true, false);
     }
 
-    public Predicate<ExpireFileEntry> dataFileSkipper(Snapshot fromSnapshot) throws Exception {
-        return dataFileSkipper(Collections.singletonList(fromSnapshot));
-    }
-
     public Predicate<ExpireFileEntry> dataFileSkipper(List<Snapshot> fromSnapshots)
             throws Exception {
         Map<BinaryRow, Map<Integer, Set<String>>> skipped = new HashMap<>();
