@@ -18,6 +18,7 @@
 
 package org.apache.paimon.table;
 
+import org.apache.paimon.CoreOptions;
 import org.apache.paimon.catalog.Catalog;
 import org.apache.paimon.catalog.CatalogLoader;
 import org.apache.paimon.catalog.CatalogLockContext;
@@ -26,6 +27,8 @@ import org.apache.paimon.catalog.CatalogSnapshotCommit;
 import org.apache.paimon.catalog.Identifier;
 import org.apache.paimon.catalog.RenamingSnapshotCommit;
 import org.apache.paimon.catalog.SnapshotCommit;
+import org.apache.paimon.operation.Lock;
+import org.apache.paimon.table.source.TableQueryAuth;
 import org.apache.paimon.tag.SnapshotLoaderImpl;
 import org.apache.paimon.utils.SnapshotLoader;
 import org.apache.paimon.utils.SnapshotManager;
@@ -33,6 +36,8 @@ import org.apache.paimon.utils.SnapshotManager;
 import javax.annotation.Nullable;
 
 import java.io.Serializable;
+import java.util.Collections;
+import java.util.Optional;
 
 /** Catalog environment in table which contains log factory, metastore client factory. */
 public class CatalogEnvironment implements Serializable {
@@ -90,13 +95,18 @@ public class CatalogEnvironment implements Serializable {
 
     @Nullable
     public SnapshotCommit snapshotCommit(SnapshotManager snapshotManager) {
-        SnapshotCommit.Factory factory;
+        SnapshotCommit snapshotCommit;
         if (catalogLoader != null && supportsVersionManagement) {
-            factory = new CatalogSnapshotCommit.Factory(catalogLoader);
+            snapshotCommit = new CatalogSnapshotCommit(catalogLoader.load(), identifier, uuid);
         } else {
-            factory = new RenamingSnapshotCommit.Factory(lockFactory, lockContext);
+            Lock lock =
+                    Optional.ofNullable(lockFactory)
+                            .map(factory -> factory.createLock(lockContext))
+                            .map(l -> Lock.fromCatalog(l, identifier))
+                            .orElseGet(Lock::empty);
+            snapshotCommit = new RenamingSnapshotCommit(snapshotManager, lock);
         }
-        return factory.create(identifier, snapshotManager);
+        return snapshotCommit;
     }
 
     @Nullable
@@ -121,4 +131,27 @@ public class CatalogEnvironment implements Serializable {
     public CatalogLoader catalogLoader() {
         return catalogLoader;
     }
+
+    public CatalogEnvironment copy(Identifier identifier) {
+        return new CatalogEnvironment(
+                identifier,
+                uuid,
+                catalogLoader,
+                lockFactory,
+                lockContext,
+                supportsVersionManagement);
+    }
+
+    public TableQueryAuth tableQueryAuth(CoreOptions options) {
+        if (!options.queryAuthEnabled() || catalogLoader == null) {
+            return select -> Collections.emptyList();
+        }
+        return select -> {
+            try (Catalog catalog = catalogLoader.load()) {
+                return catalog.authTableQuery(identifier, select);
+            } catch (Exception e) {
+                throw new RuntimeException(e);
+            }
+        };
+    }
 }
