@@ -105,7 +105,10 @@ public class ExpireChangelogImpl implements ExpireSnapshots {
         }
 
         Preconditions.checkArgument(
-                retainMax >= retainMin, "retainMax must greater than retainMin.");
+                retainMax >= retainMin,
+                String.format(
+                        "retainMax (%s) must not be less than retainMin (%s).",
+                        retainMax, retainMin));
 
         // the min snapshot to retain from 'changelog.num-retained.max'
         // (the maximum number of snapshots to retain)
