@@ -21,8 +21,11 @@ package org.apache.paimon.table.source;
 import org.apache.paimon.data.BinaryRow;
 import org.apache.paimon.metrics.MetricRegistry;
 import org.apache.paimon.predicate.Predicate;
+import org.apache.paimon.types.RowType;
 import org.apache.paimon.utils.Filter;
 
+import javax.annotation.Nullable;
+
 import java.util.List;
 import java.util.Map;
 
@@ -31,6 +34,10 @@ public interface InnerTableScan extends TableScan {
 
     InnerTableScan withFilter(Predicate predicate);
 
+    default InnerTableScan withReadType(@Nullable RowType readType) {
+        return this;
+    }
+
     default InnerTableScan withLimit(int limit) {
         return this;
     }
@@ -59,7 +66,8 @@ public interface InnerTableScan extends TableScan {
         return this;
     }
 
-    default InnerTableScan withMetricsRegistry(MetricRegistry metricRegistry) {
+    @Override
+    default InnerTableScan withMetricRegistry(MetricRegistry metricRegistry) {
         // do nothing, should implement this if need
         return this;
     }
