@@ -26,13 +26,15 @@ import org.apache.paimon.fs.Path;
 import org.apache.paimon.manifest.IndexManifestEntry;
 import org.apache.paimon.manifest.ManifestEntry;
 import org.apache.paimon.manifest.ManifestFileMeta;
-import org.apache.paimon.operation.DefaultValueAssigner;
 import org.apache.paimon.schema.SchemaManager;
 import org.apache.paimon.table.DataTable;
+import org.apache.paimon.table.FallbackReadFileStoreTable;
+import org.apache.paimon.table.FallbackReadFileStoreTable.FallbackReadScan;
 import org.apache.paimon.table.FileStoreTable;
 import org.apache.paimon.table.ReadonlyTable;
 import org.apache.paimon.table.Table;
 import org.apache.paimon.table.source.DataTableBatchScan;
+import org.apache.paimon.table.source.DataTableScan;
 import org.apache.paimon.table.source.DataTableStreamScan;
 import org.apache.paimon.table.source.InnerTableRead;
 import org.apache.paimon.table.source.StreamDataTableScan;
@@ -48,7 +50,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Optional;
 
-import static org.apache.paimon.catalog.Catalog.SYSTEM_TABLE_SPLITTER;
+import static org.apache.paimon.catalog.Identifier.SYSTEM_TABLE_SPLITTER;
 
 /**
  * A {@link Table} optimized for reading by avoiding merging files.
@@ -121,6 +123,10 @@ public class ReadOptimizedTable implements DataTable, ReadonlyTable {
 
     @Override
     public SnapshotReader newSnapshotReader() {
+        return newSnapshotReader(wrapped);
+    }
+
+    private SnapshotReader newSnapshotReader(FileStoreTable wrapped) {
         if (!wrapped.schema().primaryKeys().isEmpty()) {
             return wrapped.newSnapshotReader()
                     .withLevel(coreOptions().numLevels() - 1)
@@ -131,12 +137,21 @@ public class ReadOptimizedTable implements DataTable, ReadonlyTable {
     }
 
     @Override
-    public DataTableBatchScan newScan() {
+    public DataTableScan newScan() {
+        if (wrapped instanceof FallbackReadFileStoreTable) {
+            FallbackReadFileStoreTable table = (FallbackReadFileStoreTable) wrapped;
+            return new FallbackReadScan(newScan(table.wrapped()), newScan(table.fallback()));
+        }
+        return newScan(wrapped);
+    }
+
+    private DataTableScan newScan(FileStoreTable wrapped) {
+        CoreOptions options = wrapped.coreOptions();
         return new DataTableBatchScan(
-                !wrapped.schema().primaryKeys().isEmpty(),
-                coreOptions(),
-                newSnapshotReader(),
-                DefaultValueAssigner.create(wrapped.schema()));
+                wrapped.schema(),
+                options,
+                newSnapshotReader(wrapped),
+                wrapped.catalogEnvironment().tableQueryAuth(options));
     }
 
     @Override
@@ -146,12 +161,14 @@ public class ReadOptimizedTable implements DataTable, ReadonlyTable {
                     "Unsupported streaming scan for read optimized table");
         }
         return new DataTableStreamScan(
+                wrapped.schema(),
                 coreOptions(),
                 newSnapshotReader(),
                 snapshotManager(),
                 changelogManager(),
                 wrapped.supportStreamingReadOverwrite(),
-                DefaultValueAssigner.create(wrapped.schema()));
+                wrapped.catalogEnvironment().tableQueryAuth(coreOptions()),
+                !wrapped.schema().primaryKeys().isEmpty());
     }
 
     @Override
