@@ -360,29 +360,6 @@ public abstract class MergeTreeTestBase {
         doTestWriteRead(3, 20_000);
     }
 
-    @Test
-    public void testChangelog() throws Exception {
-        writer =
-                createMergeTreeWriter(
-                        Collections.emptyList(),
-                        createCompactManager(service, Collections.emptyList()),
-                        ChangelogProducer.INPUT);
-
-        doTestWriteReadWithChangelog(8, 200, false);
-    }
-
-    @Test
-    public void testChangelogFromCopyingData() throws Exception {
-        writer =
-                createMergeTreeWriter(
-                        Collections.emptyList(),
-                        createCompactManager(service, Collections.emptyList()),
-                        ChangelogProducer.INPUT);
-        writer.withInsertOnly(true);
-
-        doTestWriteReadWithChangelog(8, 200, true);
-    }
-
     private void doTestWriteRead(int batchNumber) throws Exception {
         doTestWriteRead(batchNumber, 200);
     }
@@ -429,77 +406,12 @@ public abstract class MergeTreeTestBase {
         assertThat(files).isEqualTo(Collections.emptySet());
     }
 
-    private void doTestWriteReadWithChangelog(
-            int batchNumber, int perBatch, boolean isChangelogEqualToData) throws Exception {
-        List<TestRecord> expected = new ArrayList<>();
-        List<DataFileMeta> newFiles = new ArrayList<>();
-        List<DataFileMeta> changelogFiles = new ArrayList<>();
-        Set<String> newFileNames = new HashSet<>();
-        List<DataFileMeta> compactedFiles = new ArrayList<>();
-
-        // write batch and commit
-        for (int i = 0; i <= batchNumber; i++) {
-            if (i < batchNumber) {
-                expected.addAll(writeBatch(perBatch));
-            } else {
-                writer.sync();
-            }
-
-            CommitIncrement increment = writer.prepareCommit(true);
-            newFiles.addAll(increment.newFilesIncrement().newFiles());
-            changelogFiles.addAll(increment.newFilesIncrement().changelogFiles());
-            mergeCompacted(newFileNames, compactedFiles, increment);
-        }
-
-        // assert records from writer
-        assertRecords(expected);
-
-        // assert records from increment new files
-        assertRecords(expected, newFiles, false);
-        assertRecords(expected, newFiles, true);
-
-        // assert records from changelog files
-        if (isChangelogEqualToData) {
-            assertRecords(expected, changelogFiles, false);
-            assertRecords(expected, changelogFiles, true);
-        } else {
-            List<TestRecord> actual = new ArrayList<>();
-            for (DataFileMeta changelogFile : changelogFiles) {
-                actual.addAll(readAll(Collections.singletonList(changelogFile), false));
-            }
-            assertThat(actual).containsExactlyInAnyOrder(expected.toArray(new TestRecord[0]));
-        }
-
-        // assert records from increment compacted files
-        assertRecords(expected, compactedFiles, true);
-
-        writer.close();
-
-        Path bucketDir = writerFactory.pathFactory(0).newPath().getParent();
-        Set<String> files =
-                Arrays.stream(LocalFileIO.create().listStatus(bucketDir))
-                        .map(FileStatus::getPath)
-                        .map(Path::getName)
-                        .collect(Collectors.toSet());
-        newFiles.stream().map(DataFileMeta::fileName).forEach(files::remove);
-        changelogFiles.stream().map(DataFileMeta::fileName).forEach(files::remove);
-        compactedFiles.stream().map(DataFileMeta::fileName).forEach(files::remove);
-        assertThat(files).isEqualTo(Collections.emptySet());
-    }
-
     private MergeTreeWriter createMergeTreeWriter(List<DataFileMeta> files) {
         return createMergeTreeWriter(files, createCompactManager(service, files));
     }
 
     private MergeTreeWriter createMergeTreeWriter(
             List<DataFileMeta> files, MergeTreeCompactManager compactManager) {
-        return createMergeTreeWriter(files, compactManager, ChangelogProducer.NONE);
-    }
-
-    private MergeTreeWriter createMergeTreeWriter(
-            List<DataFileMeta> files,
-            MergeTreeCompactManager compactManager,
-            ChangelogProducer changelogProducer) {
         long maxSequenceNumber =
                 files.stream().map(DataFileMeta::maxSequenceNumber).max(Long::compare).orElse(-1L);
         MergeTreeWriter writer =
@@ -515,7 +427,7 @@ public abstract class MergeTreeTestBase {
                         DeduplicateMergeFunction.factory().create(),
                         writerFactory,
                         options.commitForceCompact(),
-                        changelogProducer,
+                        ChangelogProducer.NONE,
                         null,
                         null);
         writer.setMemoryPool(
@@ -542,7 +454,8 @@ public abstract class MergeTreeTestBase {
                 null,
                 false,
                 options.needLookup(),
-                null);
+                null,
+                false);
     }
 
     static class MockFailResultCompactionManager extends MergeTreeCompactManager {
@@ -566,7 +479,8 @@ public abstract class MergeTreeTestBase {
                     null,
                     false,
                     false,
-                    null);
+                    null,
+                    false);
         }
 
         protected CompactResult obtainCompactResult()
