@@ -132,6 +132,8 @@ public class CdcSinkBuilder<T> {
             case HASH_DYNAMIC:
                 return new CdcDynamicBucketSink((FileStoreTable) table)
                         .build(converted, parallelism);
+            case POSTPONE_MODE:
+                return buildForPostponeBucket(converted);
             case BUCKET_UNAWARE:
                 return buildForUnawareBucket(converted);
             default:
@@ -146,9 +148,19 @@ public class CdcSinkBuilder<T> {
         return new CdcFixedBucketSink(dataTable).sinkFrom(partitioned);
     }
 
+    private DataStreamSink<?> buildForPostponeBucket(DataStream<CdcRecord> parsed) {
+        FileStoreTable dataTable = (FileStoreTable) table;
+        DataStream<CdcRecord> partitioned =
+                partition(
+                        parsed,
+                        new CdcPostponeBucketChannelComputer(dataTable.schema()),
+                        parallelism);
+        return new CdcFixedBucketSink(dataTable).sinkFrom(partitioned);
+    }
+
     private DataStreamSink<?> buildForUnawareBucket(DataStream<CdcRecord> parsed) {
         FileStoreTable dataTable = (FileStoreTable) table;
         // rebalance it to make sure schema change work to avoid infinite loop
-        return new CdcUnawareBucketSink(dataTable, parallelism).sinkFrom(parsed.rebalance());
+        return new CdcAppendTableSink(dataTable, parallelism).sinkFrom(parsed.rebalance());
     }
 }
