@@ -67,6 +67,8 @@ public class FlinkCdcSyncDatabaseSinkBuilder<T> {
     private List<FileStoreTable> tables = new ArrayList<>();
 
     @Nullable private Integer parallelism;
+    private double writerCpu;
+    @Nullable private MemorySize writerMemory;
     private double committerCpu;
     @Nullable private MemorySize committerMemory;
 
@@ -107,6 +109,8 @@ public class FlinkCdcSyncDatabaseSinkBuilder<T> {
 
     public FlinkCdcSyncDatabaseSinkBuilder<T> withTableOptions(Options options) {
         this.parallelism = options.get(FlinkConnectorOptions.SINK_PARALLELISM);
+        this.writerCpu = options.get(FlinkConnectorOptions.SINK_WRITER_CPU);
+        this.writerMemory = options.get(FlinkConnectorOptions.SINK_WRITER_MEMORY);
         this.committerCpu = options.get(FlinkConnectorOptions.SINK_COMMITTER_CPU);
         this.committerMemory = options.get(FlinkConnectorOptions.SINK_COMMITTER_MEMORY);
         this.commitUser = createCommitUser(options);
@@ -190,6 +194,8 @@ public class FlinkCdcSyncDatabaseSinkBuilder<T> {
         FlinkCdcMultiTableSink sink =
                 new FlinkCdcMultiTableSink(
                         catalogLoader,
+                        writerCpu,
+                        writerMemory,
                         committerCpu,
                         committerMemory,
                         commitUser,
@@ -204,9 +210,16 @@ public class FlinkCdcSyncDatabaseSinkBuilder<T> {
         new CdcFixedBucketSink(table).sinkFrom(partitioned);
     }
 
+    private void buildForPostponeBucket(FileStoreTable table, DataStream<CdcRecord> parsed) {
+        DataStream<CdcRecord> partitioned =
+                partition(
+                        parsed, new CdcPostponeBucketChannelComputer(table.schema()), parallelism);
+        new CdcFixedBucketSink(table).sinkFrom(partitioned);
+    }
+
     private void buildForUnawareBucket(FileStoreTable table, DataStream<CdcRecord> parsed) {
         // rebalance it to make sure schema change work to avoid infinite loop
-        new CdcUnawareBucketSink(table, parallelism).sinkFrom(parsed.rebalance());
+        new CdcAppendTableSink(table, parallelism).sinkFrom(parsed.rebalance());
     }
 
     private void buildDividedCdcSink() {
@@ -251,6 +264,9 @@ public class FlinkCdcSyncDatabaseSinkBuilder<T> {
                 case HASH_DYNAMIC:
                     new CdcDynamicBucketSink(table).build(converted, parallelism);
                     break;
+                case POSTPONE_MODE:
+                    buildForPostponeBucket(table, converted);
+                    break;
                 case BUCKET_UNAWARE:
                     buildForUnawareBucket(table, converted);
                     break;
