@@ -130,6 +130,16 @@ public class CdcActionITCaseBase extends ActionITCaseBase {
     protected void waitForResult(
             List<String> expected, FileStoreTable table, RowType rowType, List<String> primaryKeys)
             throws Exception {
+        waitForResult(false, expected, table, rowType, primaryKeys);
+    }
+
+    protected void waitForResult(
+            boolean withRegx,
+            List<String> expected,
+            FileStoreTable table,
+            RowType rowType,
+            List<String> primaryKeys)
+            throws Exception {
         assertThat(table.schema().primaryKeys()).isEqualTo(primaryKeys);
 
         // wait for table schema to become our expected schema
@@ -165,7 +175,8 @@ public class CdcActionITCaseBase extends ActionITCaseBase {
                             rowType);
             List<String> sortedActual = new ArrayList<>(result);
             Collections.sort(sortedActual);
-            if (sortedExpected.equals(sortedActual)) {
+            if (withRegx && isRegxMatchList(sortedActual, sortedExpected)
+                    || sortedExpected.equals(sortedActual)) {
                 break;
             }
             LOG.info("actual: " + sortedActual);
@@ -174,6 +185,20 @@ public class CdcActionITCaseBase extends ActionITCaseBase {
         }
     }
 
+    private boolean isRegxMatchList(List<String> actual, List<String> expected) {
+        if (actual.size() != expected.size()) {
+            return false;
+        }
+
+        for (int i = 0; i < actual.size(); i++) {
+            if (!actual.get(i).matches(expected.get(i))) {
+                return false;
+            }
+        }
+
+        return true;
+    }
+
     protected Map<String, String> getBasicTableConfig() {
         Map<String, String> config = new HashMap<>();
         ThreadLocalRandom random = ThreadLocalRandom.current();
@@ -392,6 +417,7 @@ public class CdcActionITCaseBase extends ActionITCaseBase {
         private final List<String> partitionKeys = new ArrayList<>();
         private final List<String> primaryKeys = new ArrayList<>();
         private final List<String> metadataColumn = new ArrayList<>();
+        private final List<String> computedColumnArgs = new ArrayList<>();
         protected Map<String, String> partitionKeyMultiple = new HashMap<>();
 
         public SyncDatabaseActionBuilder(Class<T> clazz, Map<String, String> sourceConfig) {
@@ -464,6 +490,12 @@ public class CdcActionITCaseBase extends ActionITCaseBase {
             return this;
         }
 
+        public SyncDatabaseActionBuilder<T> withComputedColumnArgs(
+                List<String> computedColumnArgs) {
+            this.computedColumnArgs.addAll(computedColumnArgs);
+            return this;
+        }
+
         public SyncDatabaseActionBuilder<T> withPartitionKeyMultiple(
                 Map<String, String> partitionKeyMultiple) {
             if (partitionKeyMultiple != null) {
@@ -500,6 +532,8 @@ public class CdcActionITCaseBase extends ActionITCaseBase {
             args.addAll(listToArgs("--primary-keys", primaryKeys));
             args.addAll(listToArgs("--metadata-column", metadataColumn));
 
+            args.addAll(listToMultiArgs("--computed-column", computedColumnArgs));
+
             return createAction(clazz, args);
         }
     }
