@@ -29,8 +29,12 @@ import org.apache.paimon.catalog.PropertyChange;
 import org.apache.paimon.flink.procedure.ProcedureUtil;
 import org.apache.paimon.flink.utils.FlinkCatalogPropertiesUtil;
 import org.apache.paimon.flink.utils.FlinkDescriptorProperties;
+import org.apache.paimon.function.FunctionChange;
+import org.apache.paimon.function.FunctionDefinition;
+import org.apache.paimon.function.FunctionImpl;
 import org.apache.paimon.manifest.PartitionEntry;
 import org.apache.paimon.options.Options;
+import org.apache.paimon.schema.NestedSchemaUtils;
 import org.apache.paimon.schema.Schema;
 import org.apache.paimon.schema.SchemaChange;
 import org.apache.paimon.schema.SchemaManager;
@@ -40,8 +44,6 @@ import org.apache.paimon.table.FormatTable;
 import org.apache.paimon.table.Table;
 import org.apache.paimon.table.sink.BatchTableCommit;
 import org.apache.paimon.table.source.ReadBuilder;
-import org.apache.paimon.types.DataField;
-import org.apache.paimon.types.DataTypeRoot;
 import org.apache.paimon.utils.FileStorePathFactory;
 import org.apache.paimon.utils.InternalRowPartitionComputer;
 import org.apache.paimon.utils.Preconditions;
@@ -49,11 +51,14 @@ import org.apache.paimon.utils.StringUtils;
 import org.apache.paimon.view.View;
 import org.apache.paimon.view.ViewImpl;
 
+import org.apache.paimon.shade.guava30.com.google.common.collect.ImmutableList;
+
 import org.apache.flink.table.catalog.AbstractCatalog;
 import org.apache.flink.table.catalog.CatalogBaseTable;
 import org.apache.flink.table.catalog.CatalogDatabase;
 import org.apache.flink.table.catalog.CatalogDatabaseImpl;
 import org.apache.flink.table.catalog.CatalogFunction;
+import org.apache.flink.table.catalog.CatalogFunctionImpl;
 import org.apache.flink.table.catalog.CatalogMaterializedTable;
 import org.apache.flink.table.catalog.CatalogPartition;
 import org.apache.flink.table.catalog.CatalogPartitionImpl;
@@ -61,6 +66,7 @@ import org.apache.flink.table.catalog.CatalogPartitionSpec;
 import org.apache.flink.table.catalog.CatalogTable;
 import org.apache.flink.table.catalog.CatalogView;
 import org.apache.flink.table.catalog.Column;
+import org.apache.flink.table.catalog.FunctionLanguage;
 import org.apache.flink.table.catalog.IntervalFreshness;
 import org.apache.flink.table.catalog.ObjectPath;
 import org.apache.flink.table.catalog.ResolvedCatalogBaseTable;
@@ -89,6 +95,7 @@ import org.apache.flink.table.catalog.exceptions.CatalogException;
 import org.apache.flink.table.catalog.exceptions.DatabaseAlreadyExistException;
 import org.apache.flink.table.catalog.exceptions.DatabaseNotEmptyException;
 import org.apache.flink.table.catalog.exceptions.DatabaseNotExistException;
+import org.apache.flink.table.catalog.exceptions.FunctionAlreadyExistException;
 import org.apache.flink.table.catalog.exceptions.FunctionNotExistException;
 import org.apache.flink.table.catalog.exceptions.PartitionAlreadyExistsException;
 import org.apache.flink.table.catalog.exceptions.PartitionNotExistException;
@@ -101,6 +108,8 @@ import org.apache.flink.table.catalog.stats.CatalogTableStatistics;
 import org.apache.flink.table.expressions.Expression;
 import org.apache.flink.table.factories.Factory;
 import org.apache.flink.table.procedures.Procedure;
+import org.apache.flink.table.resource.ResourceType;
+import org.apache.flink.table.resource.ResourceUri;
 import org.apache.flink.table.types.logical.RowType;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -112,13 +121,11 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
-import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Objects;
 import java.util.Optional;
-import java.util.Set;
 import java.util.function.Function;
 import java.util.stream.Collectors;
 
@@ -168,6 +175,7 @@ import static org.apache.paimon.utils.Preconditions.checkNotNull;
 public class FlinkCatalog extends AbstractCatalog {
 
     public static final String DIALECT = "flink";
+    public static final String FUNCTION_DEFINITION_NAME = "flink";
 
     private static final Logger LOG = LoggerFactory.getLogger(FlinkCatalog.class);
 
@@ -661,132 +669,7 @@ public class FlinkCatalog extends AbstractCatalog {
             org.apache.paimon.types.DataType oldType,
             org.apache.paimon.types.DataType newType,
             List<SchemaChange> schemaChanges) {
-        String joinedNames = String.join(".", fieldNames);
-        if (oldType.getTypeRoot() == DataTypeRoot.ROW) {
-            Preconditions.checkArgument(
-                    newType.getTypeRoot() == DataTypeRoot.ROW,
-                    "Column %s can only be updated to row type, and cannot be updated to %s type",
-                    joinedNames,
-                    newType.getTypeRoot());
-            org.apache.paimon.types.RowType oldRowType = (org.apache.paimon.types.RowType) oldType;
-            org.apache.paimon.types.RowType newRowType = (org.apache.paimon.types.RowType) newType;
-
-            // check that existing fields have same order
-            Map<String, Integer> oldFieldOrders = new HashMap<>();
-            for (int i = 0; i < oldRowType.getFieldCount(); i++) {
-                oldFieldOrders.put(oldRowType.getFields().get(i).name(), i);
-            }
-            int lastIdx = -1;
-            String lastFieldName = "";
-            for (DataField newField : newRowType.getFields()) {
-                String name = newField.name();
-                if (oldFieldOrders.containsKey(name)) {
-                    int idx = oldFieldOrders.get(name);
-                    Preconditions.checkState(
-                            lastIdx < idx,
-                            "Order of existing fields in column %s must be kept the same. "
-                                    + "However, field %s and %s have changed their orders.",
-                            joinedNames,
-                            lastFieldName,
-                            name);
-                    lastIdx = idx;
-                    lastFieldName = name;
-                }
-            }
-
-            // drop fields
-            Set<String> newFieldNames = new HashSet<>(newRowType.getFieldNames());
-            for (String name : oldRowType.getFieldNames()) {
-                if (!newFieldNames.contains(name)) {
-                    List<String> dropColumnNames = new ArrayList<>(fieldNames);
-                    dropColumnNames.add(name);
-                    schemaChanges.add(
-                            SchemaChange.dropColumn(dropColumnNames.toArray(new String[0])));
-                }
-            }
-
-            for (int i = 0; i < newRowType.getFieldCount(); i++) {
-                DataField field = newRowType.getFields().get(i);
-                String name = field.name();
-                List<String> fullFieldNames = new ArrayList<>(fieldNames);
-                fullFieldNames.add(name);
-                if (!oldFieldOrders.containsKey(name)) {
-                    // add fields
-                    SchemaChange.Move move;
-                    if (i == 0) {
-                        move = SchemaChange.Move.first(name);
-                    } else {
-                        String lastName = newRowType.getFields().get(i - 1).name();
-                        move = SchemaChange.Move.after(name, lastName);
-                    }
-                    schemaChanges.add(
-                            SchemaChange.addColumn(
-                                    fullFieldNames.toArray(new String[0]),
-                                    field.type(),
-                                    field.description(),
-                                    move));
-                } else {
-                    // update existing fields
-                    DataField oldField = oldRowType.getFields().get(oldFieldOrders.get(name));
-                    if (!Objects.equals(oldField.description(), field.description())) {
-                        schemaChanges.add(
-                                SchemaChange.updateColumnComment(
-                                        fullFieldNames.toArray(new String[0]),
-                                        field.description()));
-                    }
-                    generateNestedColumnUpdates(
-                            fullFieldNames, oldField.type(), field.type(), schemaChanges);
-                }
-            }
-        } else if (oldType.getTypeRoot() == DataTypeRoot.ARRAY) {
-            Preconditions.checkArgument(
-                    newType.getTypeRoot() == DataTypeRoot.ARRAY,
-                    "Column %s can only be updated to array type, and cannot be updated to %s type",
-                    joinedNames,
-                    newType);
-            List<String> fullFieldNames = new ArrayList<>(fieldNames);
-            // add a dummy column name indicating the element of array
-            fullFieldNames.add("element");
-            generateNestedColumnUpdates(
-                    fullFieldNames,
-                    ((org.apache.paimon.types.ArrayType) oldType).getElementType(),
-                    ((org.apache.paimon.types.ArrayType) newType).getElementType(),
-                    schemaChanges);
-        } else if (oldType.getTypeRoot() == DataTypeRoot.MAP) {
-            Preconditions.checkArgument(
-                    newType.getTypeRoot() == DataTypeRoot.MAP,
-                    "Column %s can only be updated to map type, and cannot be updated to %s type",
-                    joinedNames,
-                    newType);
-            org.apache.paimon.types.MapType oldMapType = (org.apache.paimon.types.MapType) oldType;
-            org.apache.paimon.types.MapType newMapType = (org.apache.paimon.types.MapType) newType;
-            Preconditions.checkArgument(
-                    oldMapType.getKeyType().equals(newMapType.getKeyType()),
-                    "Cannot update key type of column %s from %s type to %s type",
-                    joinedNames,
-                    oldMapType.getKeyType(),
-                    newMapType.getKeyType());
-            List<String> fullFieldNames = new ArrayList<>(fieldNames);
-            // add a dummy column name indicating the value of map
-            fullFieldNames.add("value");
-            generateNestedColumnUpdates(
-                    fullFieldNames,
-                    oldMapType.getValueType(),
-                    newMapType.getValueType(),
-                    schemaChanges);
-        } else {
-            if (!oldType.equalsIgnoreNullable(newType)) {
-                schemaChanges.add(
-                        SchemaChange.updateColumnType(
-                                fieldNames.toArray(new String[0]), newType, false));
-            }
-        }
-
-        if (oldType.isNullable() != newType.isNullable()) {
-            schemaChanges.add(
-                    SchemaChange.updateColumnNullability(
-                            fieldNames.toArray(new String[0]), newType.isNullable()));
-        }
+        NestedSchemaUtils.generateNestedColumnUpdates(fieldNames, oldType, newType, schemaChanges);
     }
 
     /**
@@ -1460,40 +1343,124 @@ public class FlinkCatalog extends AbstractCatalog {
 
     @Override
     public final List<String> listFunctions(String dbName) throws CatalogException {
-        return Collections.emptyList();
+        try {
+            return catalog.listFunctions(dbName);
+        } catch (Catalog.DatabaseNotExistException e) {
+            throw new CatalogException(e.getMessage(), e);
+        }
     }
 
     @Override
     public final CatalogFunction getFunction(ObjectPath functionPath)
             throws FunctionNotExistException, CatalogException {
-        throw new FunctionNotExistException(getName(), functionPath);
+        try {
+            org.apache.paimon.function.Function function =
+                    catalog.getFunction(toIdentifier(functionPath));
+            FunctionDefinition functionDefinition = function.definition(FUNCTION_DEFINITION_NAME);
+            // as current only support file function, so check type
+            if (functionDefinition instanceof FunctionDefinition.FileFunctionDefinition) {
+                FunctionDefinition.FileFunctionDefinition fileFunctionDefinition =
+                        (FunctionDefinition.FileFunctionDefinition) functionDefinition;
+                List<ResourceUri> resourceUris =
+                        fileFunctionDefinition.fileResources().stream()
+                                .map(
+                                        resource ->
+                                                new ResourceUri(
+                                                        ResourceType.valueOf(
+                                                                resource.resourceType()),
+                                                        resource.uri()))
+                                .collect(Collectors.toList());
+                return new CatalogFunctionImpl(
+                        fileFunctionDefinition.className(),
+                        FunctionLanguage.valueOf(fileFunctionDefinition.language()),
+                        resourceUris);
+            }
+            throw new FunctionNotExistException(getName(), functionPath);
+        } catch (Catalog.FunctionNotExistException e) {
+            throw new FunctionNotExistException(getName(), functionPath);
+        }
     }
 
     @Override
     public final boolean functionExists(ObjectPath functionPath) throws CatalogException {
-        return false;
+        try {
+            return catalog.listFunctions(functionPath.getDatabaseName())
+                    .contains(functionPath.getObjectName());
+        } catch (Catalog.DatabaseNotExistException e) {
+            throw new CatalogException(e.getMessage(), e);
+        }
     }
 
     @Override
     public final void createFunction(
             ObjectPath functionPath, CatalogFunction function, boolean ignoreIfExists)
-            throws CatalogException {
-        throw new UnsupportedOperationException(
-                "Create function is not supported,"
-                        + " maybe you can use 'CREATE TEMPORARY FUNCTION' instead.");
+            throws FunctionAlreadyExistException, CatalogException {
+        List<FunctionDefinition.FunctionFileResource> fileResources =
+                function.getFunctionResources().stream()
+                        .map(
+                                r ->
+                                        new FunctionDefinition.FunctionFileResource(
+                                                r.getResourceType().name(), r.getUri()))
+                        .collect(Collectors.toList());
+        FunctionDefinition functionDefinition =
+                FunctionDefinition.file(
+                        fileResources,
+                        function.getFunctionLanguage().name(),
+                        function.getClassName(),
+                        functionPath.getObjectName());
+        Map<String, FunctionDefinition> definitions = new HashMap<>();
+        definitions.put(FUNCTION_DEFINITION_NAME, functionDefinition);
+        org.apache.paimon.function.Function paimonFunction =
+                new FunctionImpl(toIdentifier(functionPath), definitions);
+        try {
+            catalog.createFunction(toIdentifier(functionPath), paimonFunction, ignoreIfExists);
+        } catch (Catalog.FunctionAlreadyExistException | Catalog.DatabaseNotExistException e) {
+            throw new FunctionAlreadyExistException(getName(), functionPath);
+        }
     }
 
     @Override
     public final void alterFunction(
             ObjectPath functionPath, CatalogFunction newFunction, boolean ignoreIfNotExists)
-            throws CatalogException {
-        throw new UnsupportedOperationException();
+            throws FunctionNotExistException, CatalogException {
+        try {
+            org.apache.paimon.function.Function function =
+                    catalog.getFunction(toIdentifier(functionPath));
+            FunctionDefinition.FileFunctionDefinition functionDefinition =
+                    (FunctionDefinition.FileFunctionDefinition)
+                            function.definition(FUNCTION_DEFINITION_NAME);
+            if (functionDefinition != null) {
+                FunctionDefinition newFunctionDefinition =
+                        FunctionDefinition.file(
+                                functionDefinition.fileResources(),
+                                newFunction.getFunctionLanguage().name(),
+                                newFunction.getClassName(),
+                                functionPath.getObjectName());
+                FunctionChange functionChange =
+                        FunctionChange.updateDefinition(
+                                FUNCTION_DEFINITION_NAME, newFunctionDefinition);
+                catalog.alterFunction(
+                        toIdentifier(functionPath),
+                        ImmutableList.of(functionChange),
+                        ignoreIfNotExists);
+            }
+        } catch (Catalog.FunctionNotExistException e) {
+            throw new FunctionNotExistException(getName(), functionPath);
+        } catch (Catalog.DefinitionAlreadyExistException e) {
+            throw new RuntimeException(e);
+        } catch (Catalog.DefinitionNotExistException e) {
+            throw new RuntimeException(e);
+        }
     }
 
     @Override
     public final void dropFunction(ObjectPath functionPath, boolean ignoreIfNotExists)
-            throws CatalogException {
-        throw new UnsupportedOperationException();
+            throws FunctionNotExistException, CatalogException {
+        try {
+            catalog.dropFunction(toIdentifier(functionPath), ignoreIfNotExists);
+        } catch (Catalog.FunctionNotExistException e) {
+            throw new FunctionNotExistException(getName(), functionPath);
+        }
     }
 
     @Override
