@@ -191,6 +191,13 @@ public class FlinkConnectorOptions {
                             "How many splits should assign to subtask per batch in StaticFileStoreSplitEnumerator "
                                     + "to avoid exceed `akka.framesize` limit.");
 
+    public static final ConfigOption<Integer> SCAN_MAX_SNAPSHOT_COUNT =
+            key("scan.max-snapshot.count")
+                    .intType()
+                    .defaultValue(-1)
+                    .withDescription(
+                            "The max snapshot count to scan per checkpoint. Not limited when it's negative.");
+
     public static final ConfigOption<SplitAssignMode> SCAN_SPLIT_ENUMERATOR_ASSIGN_MODE =
             key("scan.split-enumerator.mode")
                     .enumType(SplitAssignMode.class)
@@ -217,12 +224,12 @@ public class FlinkConnectorOptions {
                                     + " Note: This is dangerous and is likely to cause data errors if downstream"
                                     + " is used to calculate aggregation and the input is not complete changelog.");
 
-    public static final ConfigOption<Boolean> STREAMING_READ_SHUFFLE_BUCKET_WITH_PARTITION =
-            key("streaming-read.shuffle-bucket-with-partition")
+    public static final ConfigOption<Boolean> READ_SHUFFLE_BUCKET_WITH_PARTITION =
+            key("read.shuffle-bucket-with-partition")
                     .booleanType()
                     .defaultValue(true)
-                    .withDescription(
-                            "Whether shuffle by partition and bucket when streaming read.");
+                    .withFallbackKeys("streaming-read.shuffle-bucket-with-partition")
+                    .withDescription("Whether shuffle by partition and bucket when read.");
 
     /**
      * Weight of writer buffer in managed memory, Flink will compute the memory size for writer
@@ -335,6 +342,18 @@ public class FlinkConnectorOptions {
                     .withDescription(
                             "If true, a tag will be automatically created for the snapshot created by flink savepoint.");
 
+    public static final ConfigOption<Double> SINK_WRITER_CPU =
+            ConfigOptions.key("sink.writer-cpu")
+                    .doubleType()
+                    .defaultValue(1.0)
+                    .withDescription("Sink writer cpu to control cpu cores of writer.");
+
+    public static final ConfigOption<MemorySize> SINK_WRITER_MEMORY =
+            ConfigOptions.key("sink.writer-memory")
+                    .memoryType()
+                    .noDefaultValue()
+                    .withDescription("Sink writer memory to control heap memory of writer.");
+
     public static final ConfigOption<Double> SINK_COMMITTER_CPU =
             ConfigOptions.key("sink.committer-cpu")
                     .doubleType()
@@ -378,6 +397,13 @@ public class FlinkConnectorOptions {
                             "You can specify time interval for partition, for example, "
                                     + "daily partition is '1 d', hourly partition is '1 h'.");
 
+    public static final ConfigOption<Boolean> PARTITION_MARK_DONE_RECOVER_FROM_STATE =
+            key("partition.mark-done.recover-from-state")
+                    .booleanType()
+                    .defaultValue(true)
+                    .withDescription(
+                            "Whether trigger partition mark done when recover from state.");
+
     public static final ConfigOption<String> CLUSTERING_COLUMNS =
             key("sink.clustering.by-columns")
                     .stringType()
@@ -482,6 +508,21 @@ public class FlinkConnectorOptions {
                     .withDescription(
                             "Bucket number for the partitions compacted for the first time in postpone bucket tables.");
 
+    public static final ConfigOption<Boolean> SCAN_DEDICATED_SPLIT_GENERATION =
+            key("scan.dedicated-split-generation")
+                    .booleanType()
+                    .defaultValue(false)
+                    .withDescription(
+                            "If true, the split generation process would be performed during runtime on a Flink task, instead of on the JobManager during initialization phase.");
+
+    public static final ConfigOption<String> COMMIT_CUSTOM_LISTENERS =
+            key("commit.custom-listeners")
+                    .stringType()
+                    .defaultValue("")
+                    .withDescription(
+                            "Commit listener will be called after a successful commit. This option list custom commit "
+                                    + "listener identifiers separated by comma.");
+
     public static List<ConfigOption<?>> getOptions() {
         final Field[] fields = FlinkConnectorOptions.class.getFields();
         final List<ConfigOption<?>> list = new ArrayList<>(fields.length);
@@ -508,7 +549,10 @@ public class FlinkConnectorOptions {
         AUTO,
 
         /** Use full caching mode. */
-        FULL
+        FULL,
+
+        /** Use in-memory caching mode. */
+        MEMORY
     }
 
     /** Watermark emit strategy for scan. */
