@@ -18,7 +18,6 @@
 
 package org.apache.paimon.flink.sink;
 
-import org.apache.paimon.flink.ProcessRecordAttributesUtil;
 import org.apache.paimon.flink.utils.RuntimeContextUtils;
 import org.apache.paimon.index.BucketAssigner;
 import org.apache.paimon.index.HashBucketAssigner;
@@ -34,7 +33,6 @@ import org.apache.flink.api.java.tuple.Tuple2;
 import org.apache.flink.runtime.state.StateInitializationContext;
 import org.apache.flink.streaming.api.operators.AbstractStreamOperator;
 import org.apache.flink.streaming.api.operators.OneInputStreamOperator;
-import org.apache.flink.streaming.runtime.streamrecord.RecordAttributes;
 import org.apache.flink.streaming.runtime.streamrecord.StreamRecord;
 
 /** Assign bucket for the input record, output record with bucket. */
@@ -106,11 +104,6 @@ public class HashBucketAssignerOperator<T> extends AbstractStreamOperator<Tuple2
         output.collect(new StreamRecord<>(new Tuple2<>(value, bucket)));
     }
 
-    @Override
-    public void processRecordAttributes(RecordAttributes recordAttributes) {
-        ProcessRecordAttributesUtil.processWithOutput(recordAttributes, output);
-    }
-
     @Override
     public void prepareSnapshotPreBarrier(long checkpointId) {
         assigner.prepareCommit(checkpointId);
