@@ -23,14 +23,12 @@ import org.apache.paimon.CoreOptions.MergeEngine;
 import org.apache.paimon.flink.LogicalTypeConversion;
 import org.apache.paimon.flink.PredicateConverter;
 import org.apache.paimon.flink.log.LogStoreTableFactory;
-import org.apache.paimon.operation.FileStoreCommit;
 import org.apache.paimon.options.Options;
 import org.apache.paimon.predicate.OnlyPartitionKeyEqualVisitor;
 import org.apache.paimon.predicate.Predicate;
 import org.apache.paimon.predicate.PredicateBuilder;
-import org.apache.paimon.table.FileStoreTable;
 import org.apache.paimon.table.Table;
-import org.apache.paimon.table.sink.BatchWriteBuilder;
+import org.apache.paimon.table.sink.BatchTableCommit;
 
 import org.apache.flink.table.catalog.Column;
 import org.apache.flink.table.catalog.ObjectIdentifier;
@@ -55,12 +53,12 @@ import java.util.Optional;
 import java.util.Set;
 import java.util.stream.Collectors;
 
+import static org.apache.paimon.CoreOptions.AGGREGATION_REMOVE_RECORD_ON_DELETE;
 import static org.apache.paimon.CoreOptions.MERGE_ENGINE;
 import static org.apache.paimon.CoreOptions.MergeEngine.DEDUPLICATE;
 import static org.apache.paimon.CoreOptions.MergeEngine.PARTIAL_UPDATE;
 import static org.apache.paimon.CoreOptions.PARTIAL_UPDATE_REMOVE_RECORD_ON_DELETE;
 import static org.apache.paimon.CoreOptions.PARTIAL_UPDATE_REMOVE_RECORD_ON_SEQUENCE_GROUP;
-import static org.apache.paimon.CoreOptions.createCommitUser;
 import static org.apache.paimon.mergetree.compact.PartialUpdateMergeFunction.SEQUENCE_GROUP;
 import static org.apache.paimon.utils.Preconditions.checkArgument;
 
@@ -163,20 +161,16 @@ public abstract class SupportsRowLevelOperationFlinkTableSink extends FlinkTable
 
     @Override
     public Optional<Long> executeDeletion() {
-        FileStoreTable fileStoreTable = (FileStoreTable) table;
-        try (FileStoreCommit commit =
-                fileStoreTable
-                        .store()
-                        .newCommit(
-                                createCommitUser(fileStoreTable.coreOptions().toConfiguration()))) {
-            long identifier = BatchWriteBuilder.COMMIT_IDENTIFIER;
+        try (BatchTableCommit commit = table.newBatchWriteBuilder().newCommit()) {
             if (deletePredicate == null) {
-                commit.truncateTable(identifier);
+                commit.truncateTable();
             } else {
                 checkArgument(deleteIsDropPartition());
-                commit.dropPartitions(Collections.singletonList(deletePartitions()), identifier);
+                commit.truncatePartitions(Collections.singletonList(deletePartitions()));
             }
             return Optional.empty();
+        } catch (Exception e) {
+            throw new RuntimeException(e);
         }
     }
 
@@ -208,6 +202,16 @@ public abstract class SupportsRowLevelOperationFlinkTableSink extends FlinkTable
                                     SEQUENCE_GROUP,
                                     PARTIAL_UPDATE_REMOVE_RECORD_ON_SEQUENCE_GROUP));
                 }
+            case AGGREGATE:
+                if (options.get(AGGREGATION_REMOVE_RECORD_ON_DELETE)) {
+                    return;
+                } else {
+                    throw new UnsupportedOperationException(
+                            String.format(
+                                    "Merge engine %s doesn't support batch delete by default. To support batch delete, "
+                                            + "please set %s to true.",
+                                    mergeEngine, AGGREGATION_REMOVE_RECORD_ON_DELETE.key()));
+                }
             default:
                 throw new UnsupportedOperationException(
                         String.format(
