@@ -40,6 +40,15 @@ import static org.assertj.core.api.Assertions.assertThatThrownBy;
 /** IT cases for table with branches using SQL. */
 public class BranchSqlITCase extends CatalogITCaseBase {
 
+    @Test
+    public void testDefaultValue() throws Exception {
+        sql("CREATE TABLE T (a INT, b INT)");
+        sql("CALL sys.alter_column_default_value('default.T', 'b', '5')");
+        sql("INSERT INTO T (a) VALUES (1), (2)");
+        assertThat(collectResult("SELECT * FROM T"))
+                .containsExactlyInAnyOrder("+I[1, 5]", "+I[2, 5]");
+    }
+
     @Test
     public void testAlterBranchTable() throws Exception {
         sql(
@@ -655,6 +664,28 @@ public class BranchSqlITCase extends CatalogITCaseBase {
                         "+I[2, 20, dog, 2]");
     }
 
+    @Test
+    public void testMainAndFallbackNoPrimaryKeys() throws Exception {
+        sql("CREATE TABLE t ( pt INT, v INT ) PARTITIONED BY (pt) WITH ( 'bucket' = '-1' )");
+        sql("INSERT INTO t VALUES (1, 110)");
+        sql("CALL sys.create_branch('default.t', 'test')");
+        sql("ALTER TABLE t SET ( 'scan.fallback-branch' = 'test' )");
+        sql("INSERT INTO `t$branch_test` VALUES (2, 210)");
+        assertThat(collectResult("SELECT * FROM t"))
+                .containsExactlyInAnyOrder("+I[1, 110]", "+I[2, 210]");
+
+        sql("ALTER TABLE t ADD v2 INT");
+        sql("ALTER TABLE `t$branch_test` ADD v2 INT");
+        sql("INSERT INTO t VALUES (1, 120, 1200)");
+        sql("INSERT INTO `t$branch_test` VALUES (2, 220, 2200)");
+        assertThat(collectResult("SELECT * FROM t"))
+                .containsExactlyInAnyOrder(
+                        "+I[1, 110, null]",
+                        "+I[2, 210, null]",
+                        "+I[1, 120, 1200]",
+                        "+I[2, 220, 2200]");
+    }
+
     private List<String> collectResult(String sql) throws Exception {
         List<String> result = new ArrayList<>();
         try (CloseableIterator<Row> it = tEnv.executeSql(sql).collect()) {
