@@ -38,6 +38,7 @@ import javax.annotation.Nonnull;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.concurrent.ThreadLocalRandom;
 import java.util.stream.Collectors;
 
 import static org.apache.flink.table.api.config.TableConfigOptions.TABLE_DML_SYNC;
@@ -1096,11 +1097,56 @@ public class CatalogTableITCase extends CatalogITCaseBase {
     @Test
     public void testReadOptimizedTable() {
         sql("CREATE TABLE T (k INT, v INT, PRIMARY KEY (k) NOT ENFORCED) WITH ('bucket' = '1')");
-        innerTestReadOptimizedTable();
+        innerTestReadOptimizedTableAndCheckData("T");
 
         sql("DROP TABLE T");
         sql("CREATE TABLE T (k INT, v INT, PRIMARY KEY (k) NOT ENFORCED) WITH ('bucket' = '-1')");
-        innerTestReadOptimizedTable();
+        innerTestReadOptimizedTableAndCheckData("T");
+    }
+
+    @Test
+    public void testReadOptimizedTableFallBack() {
+        sql("CREATE TABLE T (k INT, v INT, PRIMARY KEY (k) NOT ENFORCED) WITH ('bucket' = '1')");
+        sql("CALL sys.create_branch('default.T', 'stream')");
+        sql("ALTER TABLE T SET ('scan.fallback-branch' = 'stream')");
+        innerTestReadOptimizedTableAndCheckData("T$branch_stream");
+
+        sql("DROP TABLE T");
+        sql("CREATE TABLE T (k INT, v INT, PRIMARY KEY (k) NOT ENFORCED) WITH ('bucket' = '-1')");
+        sql("CALL sys.create_branch('default.T', 'stream')");
+        sql("ALTER TABLE T SET ('scan.fallback-branch' = 'stream')");
+        innerTestReadOptimizedTableAndCheckData("T$branch_stream");
+        // main branch is append table and fallback branch is pk table.
+        sql("DROP TABLE T");
+        sql("CREATE TABLE T (k INT, v INT, n INT) PARTITIONED BY (k)");
+        sql("CALL sys.create_branch('default.T', 'stream')");
+        sql("ALTER TABLE T SET ('scan.fallback-branch' = 'stream')");
+        sql(
+                "ALTER TABLE T$branch_stream SET ('primary-key' = 'k,v', 'bucket' = '2','changelog-producer' = 'lookup')");
+        // full compaction will always be performed at the end of batch jobs, as long as
+        // full-compaction.delta-commits is set, regardless of its value
+        sql("show create table T$branch_stream");
+        sql(
+                "INSERT INTO T$branch_stream /*+ OPTIONS('full-compaction.delta-commits' = '1') */ VALUES (1, 10, 10), (2, 20, 20)");
+        List<Row> result = sql("SELECT k, v, n FROM T$ro ORDER BY k");
+        assertThat(result).containsExactly(Row.of(1, 10, 10), Row.of(2, 20, 20));
+        sql(
+                "INSERT INTO T$branch_stream /*+ OPTIONS('write-only' = 'true') */VALUES (1, 10, 11), (3, 30, 30)");
+        result = sql("SELECT k, v, n FROM T$ro ORDER BY k, v");
+        assertThat(result).containsExactly(Row.of(1, 10, 10), Row.of(2, 20, 20));
+        sql(
+                "INSERT INTO T$branch_stream /*+ OPTIONS('full-compaction.delta-commits' = '1') */ VALUES (1, 10, 12), (2, 20, 21), (3, 30, 31)");
+        result = sql("SELECT k, v, n FROM T$ro ORDER BY k, v");
+        assertThat(result).containsExactly(Row.of(1, 10, 12), Row.of(2, 20, 21), Row.of(3, 30, 31));
+        // insert into data to main branch.
+        sql("INSERT INTO T VALUES (1, 10, 101), (2222, 202, 202)");
+        result = sql("SELECT k, v, n FROM T$ro ORDER BY k");
+        assertThat(result)
+                .containsExactly(
+                        Row.of(1, 10, 101),
+                        Row.of(2, 20, 21),
+                        Row.of(3, 30, 31),
+                        Row.of(2222, 202, 202));
     }
 
     @Test
@@ -1140,8 +1186,10 @@ public class CatalogTableITCase extends CatalogITCaseBase {
     @Test
     public void testIndexesTable() {
         sql(
-                "CREATE TABLE T (pt STRING, a INT, b STRING, PRIMARY KEY (pt, a) NOT ENFORCED)"
-                        + " PARTITIONED BY (pt) with ('deletion-vectors.enabled'='true')");
+                String.format(
+                        "CREATE TABLE T (pt STRING, a INT, b STRING, PRIMARY KEY (pt, a) NOT ENFORCED)"
+                                + " PARTITIONED BY (pt) with ('deletion-vectors.enabled'='true', 'deletion-vectors.bitmap64' = '%s')",
+                        ThreadLocalRandom.current().nextBoolean()));
         sql(
                 "INSERT INTO T VALUES ('2024-10-01', 1, 'aaaaaaaaaaaaaaaaaaa'), ('2024-10-01', 2, 'b'), ('2024-10-01', 3, 'c')");
         sql("INSERT INTO T VALUES ('2024-10-01', 1, 'a_new1'), ('2024-10-01', 3, 'c_new1')");
@@ -1164,26 +1212,30 @@ public class CatalogTableITCase extends CatalogITCaseBase {
         assertThat(row.getField(1)).isEqualTo(0);
         assertThat(row.getField(2)).isEqualTo("DELETION_VECTORS");
         assertThat(row.getField(3).toString().startsWith("index-")).isTrue();
-        assertThat(row.getField(4)).isEqualTo(33L);
+        // dv version 1 and 2 have different file size
+        assertThat(row.getField(4)).isIn(33L, 45L);
         assertThat(row.getField(5)).isEqualTo(1L);
         assertThat(row.getField(6)).isNotNull();
     }
 
-    private void innerTestReadOptimizedTable() {
+    private void innerTestReadOptimizedTableAndCheckData(String insertTableName) {
         // full compaction will always be performed at the end of batch jobs, as long as
         // full-compaction.delta-commits is set, regardless of its value
         sql(
-                "INSERT INTO T /*+ OPTIONS('full-compaction.delta-commits' = '100') */ VALUES (1, 10), (2, 20)");
+                String.format(
+                        "INSERT INTO %s /*+ OPTIONS('full-compaction.delta-commits' = '100') */ VALUES (1, 10), (2, 20)",
+                        insertTableName));
         List<Row> result = sql("SELECT k, v FROM T$ro ORDER BY k");
         assertThat(result).containsExactly(Row.of(1, 10), Row.of(2, 20));
 
         // no compaction, so result of ro table does not change
-        sql("INSERT INTO T VALUES (1, 11), (3, 30)");
+        sql(String.format("INSERT INTO %s VALUES (1, 11), (3, 30)", insertTableName));
         result = sql("SELECT k, v FROM T$ro ORDER BY k");
         assertThat(result).containsExactly(Row.of(1, 10), Row.of(2, 20));
-
         sql(
-                "INSERT INTO T /*+ OPTIONS('full-compaction.delta-commits' = '100') */ VALUES (2, 21), (3, 31)");
+                String.format(
+                        "INSERT INTO %s /*+ OPTIONS('full-compaction.delta-commits' = '100') */ VALUES (2, 21), (3, 31)",
+                        insertTableName));
         result = sql("SELECT k, v FROM T$ro ORDER BY k");
         assertThat(result).containsExactly(Row.of(1, 11), Row.of(2, 21), Row.of(3, 31));
     }
