@@ -1045,24 +1045,22 @@ public class CompactDatabaseActionITCase extends CompactActionITCaseBase {
                         "--table_conf",
                         CoreOptions.SNAPSHOT_NUM_RETAINED_MAX.key() + "=3");
 
-        StreamExecutionEnvironment env =
-                streamExecutionEnvironmentBuilder().streamingMode().build();
+        StreamExecutionEnvironment env = streamExecutionEnvironmentBuilder().batchMode().build();
         action.withStreamExecutionEnvironment(env).build();
         JobClient jobClient = env.executeAsync();
 
         waitUtil(
                 () -> snapshotManager.latestSnapshotId() == 11L,
-                Duration.ofSeconds(60),
+                Duration.ofSeconds(240),
                 Duration.ofMillis(500));
-        jobClient.cancel();
 
         assertThat(snapshotManager.latestSnapshot().commitKind())
                 .isEqualTo(Snapshot.CommitKind.COMPACT);
 
         waitUtil(
                 () -> snapshotManager.earliestSnapshotId() == 9L,
-                Duration.ofSeconds(60),
-                Duration.ofMillis(200),
+                Duration.ofSeconds(240),
+                Duration.ofMillis(500),
                 "Failed to wait snapshot expiration success");
 
         List<DataSplit> splits = table.newSnapshotReader().read().dataSplits();
@@ -1072,6 +1070,7 @@ public class CompactDatabaseActionITCase extends CompactActionITCaseBase {
                 splits.get(0).dataFiles().stream()
                         .anyMatch(file -> file.fileFormat().equalsIgnoreCase("avro"));
         assertThat(hasAvroFile).isTrue();
+        jobClient.cancel();
     }
 
     private void writeData(
