@@ -24,6 +24,7 @@ import org.apache.paimon.data.BinaryRow;
 import org.apache.paimon.data.GenericRow;
 import org.apache.paimon.flink.FlinkConnectorOptions;
 import org.apache.paimon.flink.utils.TestingMetricUtils;
+import org.apache.paimon.fs.Path;
 import org.apache.paimon.fs.local.LocalFileIO;
 import org.apache.paimon.io.CompactIncrement;
 import org.apache.paimon.io.DataIncrement;
@@ -58,7 +59,6 @@ import org.apache.flink.streaming.api.watermark.Watermark;
 import org.apache.flink.streaming.util.AbstractStreamOperatorTestHarness;
 import org.apache.flink.streaming.util.OneInputStreamOperatorTestHarness;
 import org.apache.flink.util.Preconditions;
-import org.assertj.core.api.Assertions;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 
@@ -69,6 +69,7 @@ import java.util.UUID;
 
 import static org.apache.paimon.SnapshotTest.newSnapshotManager;
 import static org.assertj.core.api.Assertions.assertThat;
+import static org.assertj.core.api.Assertions.assertThatCode;
 import static org.assertj.core.api.Assertions.fail;
 
 /** Tests for {@link CommitterOperator}. */
@@ -282,9 +283,8 @@ public class CommitterOperatorTest extends CommitterOperatorTestBase {
         testHarness1.initializeState(snapshot);
         testHarness1.close();
 
-        Assertions.assertThat(actual.size()).isEqualTo(1);
-
-        Assertions.assertThat(actual).hasSameElementsAs(Lists.newArrayList(commitUser));
+        assertThat(actual.size()).isEqualTo(1);
+        assertThat(actual).hasSameElementsAs(Lists.newArrayList(commitUser));
     }
 
     @Test
@@ -325,7 +325,7 @@ public class CommitterOperatorTest extends CommitterOperatorTestBase {
         OneInputStreamOperatorTestHarness<Committable, Committable> testHarness =
                 createTestHarness(operatorFactory);
         testHarness.open();
-        Assertions.assertThatCode(
+        assertThatCode(
                         () -> {
                             long time = System.currentTimeMillis();
                             long cp = 0L;
@@ -387,7 +387,7 @@ public class CommitterOperatorTest extends CommitterOperatorTestBase {
                 .doesNotThrowAnyException();
 
         if (operatorFactory instanceof CommitterOperator) {
-            Assertions.assertThat(
+            assertThat(
                             ((ManifestCommittable)
                                             ((CommitterOperator) operatorFactory)
                                                     .committablesPerCheckpoint.get(Long.MAX_VALUE))
@@ -396,7 +396,7 @@ public class CommitterOperatorTest extends CommitterOperatorTestBase {
                     .isEqualTo(3);
         }
 
-        Assertions.assertThatCode(
+        assertThatCode(
                         () -> {
                             long time = System.currentTimeMillis();
                             long cp = 0L;
@@ -548,6 +548,49 @@ public class CommitterOperatorTest extends CommitterOperatorTestBase {
         assertThat(snapshot).isNotNull();
     }
 
+    @Test
+    public void testNotTriggerPartitionMarkDownWhenRecoverFromState() throws Exception {
+        FileStoreTable table =
+                createFileStoreTable(
+                        options -> {
+                            options.set(CoreOptions.COMMIT_FORCE_CREATE_SNAPSHOT.key(), "true");
+                            options.set(
+                                    CoreOptions.PARTITION_MARK_DONE_WHEN_END_INPUT.key(), "true");
+                            options.set(
+                                    FlinkConnectorOptions.PARTITION_IDLE_TIME_TO_DONE.key(), "1h");
+                        },
+                        Collections.singletonList("b"));
+
+        OneInputStreamOperatorTestHarness<Committable, Committable> testHarness =
+                createRecoverableTestHarness(table, false);
+        testHarness.open();
+        OperatorSubtaskState snapshotState =
+                writeAndSnapshot(table, "commitUser", 1, Long.MAX_VALUE, testHarness);
+        testHarness.close();
+
+        testHarness = createRecoverableTestHarness(table, false);
+        try {
+            // commit snapshot from state, fail intentionally
+            testHarness.initializeState(snapshotState);
+            testHarness.open();
+            fail("Expecting intentional exception");
+        } catch (Exception e) {
+            assertThat(e)
+                    .hasMessageContaining(
+                            "This exception is intentionally thrown "
+                                    + "after committing the restored checkpoints. "
+                                    + "By restarting the job we hope that "
+                                    + "writers can start writing based on these new commits.");
+        }
+
+        testHarness.notifyOfCompletedCheckpoint(Long.MAX_VALUE);
+        Snapshot snapshot = table.snapshotManager().latestSnapshot();
+        assertThat(snapshot).isNotNull();
+
+        Path successFile = new Path(table.location(), "b=10/_SUCCESS");
+        assertThat(table.fileIO().exists(successFile)).isEqualTo(false);
+    }
+
     @Test
     public void testEmptyCommitWithProcessTimeTag() throws Exception {
         FileStoreTable table =
@@ -608,7 +651,7 @@ public class CommitterOperatorTest extends CommitterOperatorTestBase {
                         Committer.createContext("", metricGroup, true, false, null, 1, 1));
         committer.commit(Collections.singletonList(manifestCommittable));
         CommitterMetrics metrics = committer.getCommitterMetrics();
-        assertThat(metrics.getNumBytesOutCounter().getCount()).isEqualTo(533);
+        assertThat(metrics.getNumBytesOutCounter().getCount()).isEqualTo(572);
         assertThat(metrics.getNumRecordsOutCounter().getCount()).isEqualTo(2);
         committer.close();
     }
@@ -622,7 +665,7 @@ public class CommitterOperatorTest extends CommitterOperatorTestBase {
                         table,
                         null,
                         new RestoreAndFailCommittableStateManager<>(
-                                ManifestCommittableSerializer::new));
+                                ManifestCommittableSerializer::new, true));
         OneInputStreamOperatorTestHarness<Committable, Committable> testHarness =
                 createTestHarness(operatorFactory);
         testHarness.open();
@@ -705,7 +748,7 @@ public class CommitterOperatorTest extends CommitterOperatorTestBase {
                         table, commitUser, new NoopCommittableStateManager());
         try (OneInputStreamOperatorTestHarness<Committable, Committable> testHarness =
                 createTestHarness(operatorFactory, 10, 10, 3)) {
-            Assertions.assertThatCode(testHarness::open)
+            assertThatCode(testHarness::open)
                     .hasMessage("Committer Operator parallelism in paimon MUST be one.");
         }
     }
@@ -716,12 +759,20 @@ public class CommitterOperatorTest extends CommitterOperatorTestBase {
 
     protected OneInputStreamOperatorTestHarness<Committable, Committable>
             createRecoverableTestHarness(FileStoreTable table) throws Exception {
+        return createRecoverableTestHarness(table, true);
+    }
+
+    private OneInputStreamOperatorTestHarness<Committable, Committable>
+            createRecoverableTestHarness(
+                    FileStoreTable table, boolean partitionMarkDownRecoverFromState)
+                    throws Exception {
         OneInputStreamOperatorFactory<Committable, Committable> operatorFactory =
                 createCommitterOperatorFactory(
                         table,
                         null,
                         new RestoreAndFailCommittableStateManager<>(
-                                ManifestCommittableSerializer::new));
+                                ManifestCommittableSerializer::new,
+                                partitionMarkDownRecoverFromState));
         return createTestHarness(operatorFactory);
     }
 
