@@ -20,14 +20,16 @@ package org.apache.parquet.hadoop;
 
 import org.apache.paimon.format.parquet.ParquetInputFile;
 import org.apache.paimon.format.parquet.ParquetInputStream;
-import org.apache.paimon.fs.FileRange;
 import org.apache.paimon.fs.VectoredReadable;
 import org.apache.paimon.utils.RoaringBitmap32;
 
 import org.apache.hadoop.fs.Path;
 import org.apache.parquet.ParquetReadOptions;
+import org.apache.parquet.Preconditions;
 import org.apache.parquet.bytes.ByteBufferInputStream;
+import org.apache.parquet.bytes.ByteBufferReleaser;
 import org.apache.parquet.bytes.BytesInput;
+import org.apache.parquet.bytes.ReusingByteBufferAllocator;
 import org.apache.parquet.column.ColumnDescriptor;
 import org.apache.parquet.column.page.DataPage;
 import org.apache.parquet.column.page.DataPageV1;
@@ -63,6 +65,7 @@ import org.apache.parquet.hadoop.metadata.ColumnPath;
 import org.apache.parquet.hadoop.metadata.FileMetaData;
 import org.apache.parquet.hadoop.metadata.ParquetMetadata;
 import org.apache.parquet.hadoop.util.counters.BenchmarkCounter;
+import org.apache.parquet.hadoop.util.wrapped.io.FutureIO;
 import org.apache.parquet.internal.column.columnindex.ColumnIndex;
 import org.apache.parquet.internal.column.columnindex.OffsetIndex;
 import org.apache.parquet.internal.filter2.columnindex.ColumnIndexFilter;
@@ -71,15 +74,17 @@ import org.apache.parquet.internal.filter2.columnindex.RowRanges;
 import org.apache.parquet.internal.hadoop.metadata.IndexReference;
 import org.apache.parquet.io.InputFile;
 import org.apache.parquet.io.ParquetDecodingException;
+import org.apache.parquet.io.ParquetFileRange;
 import org.apache.parquet.io.SeekableInputStream;
 import org.apache.parquet.schema.MessageType;
 import org.apache.parquet.schema.PrimitiveType;
-import org.apache.yetus.audience.InterfaceAudience.Private;
+import org.apache.parquet.util.AutoCloseables;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import javax.annotation.Nullable;
 
+import java.io.ByteArrayInputStream;
 import java.io.Closeable;
 import java.io.IOException;
 import java.io.InputStream;
@@ -94,11 +99,11 @@ import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Objects;
 import java.util.Set;
-import java.util.concurrent.ExecutionException;
+import java.util.concurrent.TimeUnit;
+import java.util.concurrent.TimeoutException;
 import java.util.stream.Collectors;
 import java.util.zip.CRC32;
 
-import static org.apache.paimon.utils.Preconditions.checkArgument;
 import static org.apache.parquet.bytes.BytesUtils.readIntLittleEndian;
 import static org.apache.parquet.filter2.compat.RowGroupFilter.FilterLevel.BLOOMFILTER;
 import static org.apache.parquet.filter2.compat.RowGroupFilter.FilterLevel.DICTIONARY;
@@ -118,11 +123,20 @@ public class ParquetFileReader implements Closeable {
 
     private static final Logger LOG = LoggerFactory.getLogger(ParquetFileReader.class);
 
+    public static final long HADOOP_VECTORED_READ_TIMEOUT_SECONDS = 300;
+
     private final ParquetMetadataConverter converter;
 
     private final CRC32 crc;
+    private final ReusingByteBufferAllocator crcAllocator;
 
-    private static ParquetMetadata readFooter(
+    public static final ParquetMetadata readFooter(
+            InputFile file, ParquetReadOptions options, SeekableInputStream f) throws IOException {
+        ParquetMetadataConverter converter = new ParquetMetadataConverter(options);
+        return readFooter(file, options, f, converter);
+    }
+
+    private static final ParquetMetadata readFooter(
             InputFile file,
             ParquetReadOptions options,
             SeekableInputStream f,
@@ -180,35 +194,39 @@ public class ParquetFileReader implements Closeable {
 
         // Read all the footer bytes in one time to avoid multiple read operations,
         // since it can be pretty time consuming for a single read operation in HDFS.
-        ByteBuffer footerBytesBuffer = ByteBuffer.allocate(fileMetadataLength);
-        f.readFully(footerBytesBuffer);
-        LOG.debug("Finished to read all footer bytes.");
-        footerBytesBuffer.flip();
-        InputStream footerBytesStream = ByteBufferInputStream.wrap(footerBytesBuffer);
+        ByteBuffer footerBytesBuffer = options.getAllocator().allocate(fileMetadataLength);
+        try {
+            f.readFully(footerBytesBuffer);
+            LOG.debug("Finished to read all footer bytes.");
+            footerBytesBuffer.flip();
+            InputStream footerBytesStream = ByteBufferInputStream.wrap(footerBytesBuffer);
 
-        // Regular file, or encrypted file with plaintext footer
-        if (!encryptedFooterMode) {
+            // Regular file, or encrypted file with plaintext footer
+            if (!encryptedFooterMode) {
+                return converter.readParquetMetadata(
+                        footerBytesStream,
+                        options.getMetadataFilter(),
+                        fileDecryptor,
+                        false,
+                        fileMetadataLength);
+            }
+
+            // Encrypted file with encrypted footer
+            if (null == fileDecryptor) {
+                throw new ParquetCryptoRuntimeException(
+                        "Trying to read file with encrypted footer. No keys available");
+            }
+            FileCryptoMetaData fileCryptoMetaData = readFileCryptoMetaData(footerBytesStream);
+            fileDecryptor.setFileCryptoMetaData(
+                    fileCryptoMetaData.getEncryption_algorithm(),
+                    true,
+                    fileCryptoMetaData.getKey_metadata());
+            // footer length is required only for signed plaintext footers
             return converter.readParquetMetadata(
-                    footerBytesStream,
-                    options.getMetadataFilter(),
-                    fileDecryptor,
-                    false,
-                    fileMetadataLength);
+                    footerBytesStream, options.getMetadataFilter(), fileDecryptor, true, 0);
+        } finally {
+            options.getAllocator().release(footerBytesBuffer);
         }
-
-        // Encrypted file with encrypted footer
-        if (null == fileDecryptor) {
-            throw new ParquetCryptoRuntimeException(
-                    "Trying to read file with encrypted footer. No keys available");
-        }
-        FileCryptoMetaData fileCryptoMetaData = readFileCryptoMetaData(footerBytesStream);
-        fileDecryptor.setFileCryptoMetaData(
-                fileCryptoMetaData.getEncryption_algorithm(),
-                true,
-                fileCryptoMetaData.getKey_metadata());
-        // footer length is required only for signed plaintext footers
-        return converter.readParquetMetadata(
-                footerBytesStream, options.getMetadataFilter(), fileDecryptor, true, 0);
     }
 
     protected final ParquetInputStream f;
@@ -229,7 +247,7 @@ public class ParquetFileReader implements Closeable {
     private ColumnChunkPageReadStore currentRowGroup = null;
     private DictionaryPageReader nextDictionaryReader = null;
 
-    private InternalFileDecryptor fileDecryptor;
+    private InternalFileDecryptor fileDecryptor = null;
 
     public ParquetFileReader(
             InputFile file, ParquetReadOptions options, @Nullable RoaringBitmap32 selection)
@@ -268,25 +286,20 @@ public class ParquetFileReader implements Closeable {
         for (ColumnDescriptor col : footer.getFileMetaData().getSchema().getColumns()) {
             paths.put(ColumnPath.get(col.getPath()), col);
         }
-        this.crc = options.usePageChecksumVerification() ? new CRC32() : null;
+
+        if (options.usePageChecksumVerification()) {
+            this.crc = new CRC32();
+            this.crcAllocator = ReusingByteBufferAllocator.strict(options.getAllocator());
+        } else {
+            this.crc = null;
+            this.crcAllocator = null;
+        }
     }
 
     private static <T> List<T> listWithNulls(int size) {
         return new ArrayList<>(Collections.nCopies(size, null));
     }
 
-    private boolean checkRowIndexOffsetExists(List<BlockMetaData> blocks) {
-        for (BlockMetaData block : blocks) {
-            if (block.getRowIndexOffset() == -1) {
-                LOG.warn(
-                        "Row index offset was not found in block metadata of file {}, skip applying filter in order to get the correct row position",
-                        file.getPath());
-                return false;
-            }
-        }
-        return true;
-    }
-
     public ParquetMetadata getFooter() {
         if (footer == null) {
             try {
@@ -341,35 +354,44 @@ public class ParquetFileReader implements Closeable {
 
     private List<BlockMetaData> filterRowGroups(List<BlockMetaData> blocks) throws IOException {
         FilterCompat.Filter recordFilter = options.getRecordFilter();
-        if (checkRowIndexOffsetExists(blocks)) {
-            if (FilterCompat.isFilteringRequired(recordFilter)) {
-                // set up data filters based on configured levels
-                List<RowGroupFilter.FilterLevel> levels = new ArrayList<>();
 
-                if (options.useStatsFilter()) {
-                    levels.add(STATISTICS);
-                }
+        for (BlockMetaData block : blocks) {
+            if (block.getRowIndexOffset() == -1) {
+                LOG.warn(
+                        "Row index offset was not found in block metadata of file {}, "
+                                + "skip applying filter in order to get the correct row position",
+                        file.getPath());
+                return blocks;
+            }
+        }
 
-                if (options.useDictionaryFilter()) {
-                    levels.add(DICTIONARY);
-                }
+        if (FilterCompat.isFilteringRequired(recordFilter)) {
+            // set up data filters based on configured levels
+            List<RowGroupFilter.FilterLevel> levels = new ArrayList<>();
 
-                if (options.useBloomFilter()) {
-                    levels.add(BLOOMFILTER);
-                }
-                blocks = RowGroupFilter.filterRowGroups(levels, recordFilter, blocks, this);
+            if (options.useStatsFilter()) {
+                levels.add(STATISTICS);
             }
 
-            if (selection != null) {
-                blocks =
-                        blocks.stream()
-                                .filter(
-                                        it ->
-                                                selection.intersects(
-                                                        it.getRowIndexOffset(),
-                                                        it.getRowIndexOffset() + it.getRowCount()))
-                                .collect(Collectors.toList());
+            if (options.useDictionaryFilter()) {
+                levels.add(DICTIONARY);
             }
+
+            if (options.useBloomFilter()) {
+                levels.add(BLOOMFILTER);
+            }
+            blocks = RowGroupFilter.filterRowGroups(levels, recordFilter, blocks, this);
+        }
+
+        if (selection != null) {
+            blocks =
+                    blocks.stream()
+                            .filter(
+                                    it ->
+                                            selection.intersects(
+                                                    it.getRowIndexOffset(),
+                                                    it.getRowIndexOffset() + it.getRowCount()))
+                            .collect(Collectors.toList());
         }
 
         return blocks;
@@ -398,8 +420,8 @@ public class ParquetFileReader implements Closeable {
      * Reads all the columns requested from the row group at the specified block.
      *
      * @param blockIndex the index of the requested block
-     * @throws IOException if an error occurs while reading
      * @return the PageReadStore which can provide PageReaders for each column.
+     * @throws IOException if an error occurs while reading
      */
     public PageReadStore readRowGroup(int blockIndex) throws IOException {
         return internalReadRowGroup(blockIndex);
@@ -408,8 +430,8 @@ public class ParquetFileReader implements Closeable {
     /**
      * Reads all the columns requested from the row group at the current file position.
      *
-     * @throws IOException if an error occurs while reading
      * @return the PageReadStore which can provide PageReaders for each column.
+     * @throws IOException if an error occurs while reading
      */
     public PageReadStore readNextRowGroup() throws IOException {
         ColumnChunkPageReadStore rowGroup = null;
@@ -446,7 +468,7 @@ public class ParquetFileReader implements Closeable {
         ColumnChunkPageReadStore rowGroup =
                 new ColumnChunkPageReadStore(block.getRowCount(), block.getRowIndexOffset());
         // prepare the list of consecutive parts to read them in one scan
-        List<ConsecutivePartList> allParts = new ArrayList<ConsecutivePartList>();
+        List<ConsecutivePartList> allParts = new ArrayList<>();
         ConsecutivePartList currentParts = null;
         for (ColumnChunkMetaData mc : block.getColumns()) {
             ColumnPath pathKey = mc.getPath();
@@ -466,6 +488,7 @@ public class ParquetFileReader implements Closeable {
         // actually read all the chunks
         ChunkListBuilder builder = new ChunkListBuilder(block.getRowCount());
         readAllPartsVectoredOrNormal(allParts, builder);
+        rowGroup.setReleaser(builder.releaser);
         for (Chunk chunk : builder.build()) {
             readChunkPages(chunk, block, rowGroup);
         }
@@ -560,7 +583,8 @@ public class ParquetFileReader implements Closeable {
      */
     private void readAllPartsVectoredOrNormal(
             List<ConsecutivePartList> allParts, ChunkListBuilder builder) throws IOException {
-        if (shouldUseVectoredIo()) {
+
+        if (shouldUseVectoredIo(allParts)) {
             try {
                 readVectored(allParts, builder);
                 return;
@@ -575,9 +599,35 @@ public class ParquetFileReader implements Closeable {
         }
     }
 
-    /** Should the read use vectored IO. */
-    private boolean shouldUseVectoredIo() {
-        return f.in() instanceof VectoredReadable;
+    /**
+     * Should the read use vectored IO?
+     *
+     * <p>Use Paimon vectored io.
+     *
+     * @param allParts all parts to read.
+     * @return true or false.
+     */
+    private boolean shouldUseVectoredIo(final List<ConsecutivePartList> allParts) {
+        return f.in() instanceof VectoredReadable && arePartsValidForVectoredIo(allParts);
+    }
+
+    /**
+     * Validate the parts for vectored IO. Vectored IO doesn't support reading ranges of size
+     * greater than Integer.MAX_VALUE.
+     *
+     * @param allParts all parts to read.
+     * @return true or false.
+     */
+    private boolean arePartsValidForVectoredIo(List<ConsecutivePartList> allParts) {
+        for (ConsecutivePartList consecutivePart : allParts) {
+            if (consecutivePart.length >= Integer.MAX_VALUE) {
+                LOG.debug(
+                        "Part length {} greater than Integer.MAX_VALUE thus disabling vectored IO",
+                        consecutivePart.length);
+                return false;
+            }
+        }
+        return true;
     }
 
     /**
@@ -597,27 +647,27 @@ public class ParquetFileReader implements Closeable {
      * @throws IllegalArgumentException arguments are invalid.
      * @throws UnsupportedOperationException if the filesystem does not support vectored IO.
      */
-    @SuppressWarnings("checkstyle:JavadocParagraph")
     private void readVectored(List<ConsecutivePartList> allParts, ChunkListBuilder builder)
             throws IOException {
-        List<FileRange> ranges = new ArrayList<>(allParts.size());
+
+        List<ParquetFileRange> ranges = new ArrayList<>(allParts.size());
         long totalSize = 0;
         for (ConsecutivePartList consecutiveChunks : allParts) {
             final long len = consecutiveChunks.length;
-            checkArgument(
+            Preconditions.checkArgument(
                     len < Integer.MAX_VALUE,
                     "Invalid length %s for vectored read operation. It must be less than max integer value.",
                     len);
-            ranges.add(FileRange.createFileRange(consecutiveChunks.offset, (int) len));
+            ranges.add(new ParquetFileRange(consecutiveChunks.offset, (int) len));
             totalSize += len;
         }
         LOG.debug(
                 "Reading {} bytes of data with vectored IO in {} ranges", totalSize, ranges.size());
         // Request a vectored read;
-        ((VectoredReadable) f.in()).readVectored(ranges);
+        f.readVectored(ranges, options.getAllocator());
         int k = 0;
         for (ConsecutivePartList consecutivePart : allParts) {
-            FileRange currRange = ranges.get(k++);
+            ParquetFileRange currRange = ranges.get(k++);
             consecutivePart.readFromVectoredRange(currRange, builder);
         }
     }
@@ -705,8 +755,8 @@ public class ParquetFileReader implements Closeable {
                 }
             }
         }
-        // actually read all the chunks
         readAllPartsVectoredOrNormal(allParts, builder);
+        rowGroup.setReleaser(builder.releaser);
         for (Chunk chunk : builder.build()) {
             readChunkPages(chunk, block, rowGroup);
         }
@@ -776,6 +826,10 @@ public class ParquetFileReader implements Closeable {
 
         // update the current block and instantiate a dictionary reader for it
         ++currentBlock;
+
+        if (nextDictionaryReader != null) {
+            nextDictionaryReader.close();
+        }
         this.nextDictionaryReader = null;
 
         return true;
@@ -798,11 +852,11 @@ public class ParquetFileReader implements Closeable {
         if (blockIndex < 0 || blockIndex >= blocks.size()) {
             return null;
         }
-        return new DictionaryPageReader(this, blocks.get(blockIndex));
+        return new DictionaryPageReader(this, blocks.get(blockIndex), options.getAllocator());
     }
 
     public DictionaryPageReader getDictionaryReader(BlockMetaData block) {
-        return new DictionaryPageReader(this, block);
+        return new DictionaryPageReader(this, block, options.getAllocator());
     }
 
     /**
@@ -888,10 +942,7 @@ public class ParquetFileReader implements Closeable {
         int uncompressedPageSize = pageHeader.getUncompressed_page_size();
         int compressedPageSize = pageHeader.getCompressed_page_size();
 
-        byte[] dictPageBytes = new byte[compressedPageSize];
-        fin.readFully(dictPageBytes);
-
-        BytesInput bin = BytesInput.from(dictPageBytes);
+        BytesInput bin = BytesInput.from(fin, compressedPageSize);
 
         if (null != pageDecryptor) {
             bin = BytesInput.from(pageDecryptor.decrypt(bin.toByteArray(), dictionaryPageAAD));
@@ -954,12 +1005,25 @@ public class ParquetFileReader implements Closeable {
             }
         }
 
-        // Read Bloom filter data header.
+        // Seek to Bloom filter offset.
         f.seek(bloomFilterOffset);
+
+        // Read Bloom filter length.
+        int bloomFilterLength = meta.getBloomFilterLength();
+
+        // If it is set, read Bloom filter header and bitset together.
+        // Otherwise, read Bloom filter header first and then bitset.
+        InputStream in = f;
+        if (bloomFilterLength > 0) {
+            byte[] headerAndBitSet = new byte[bloomFilterLength];
+            f.readFully(headerAndBitSet);
+            in = new ByteArrayInputStream(headerAndBitSet);
+        }
+
         BloomFilterHeader bloomFilterHeader;
         try {
             bloomFilterHeader =
-                    Util.readBloomFilterHeader(f, bloomFilterDecryptor, bloomFilterHeaderAAD);
+                    Util.readBloomFilterHeader(in, bloomFilterDecryptor, bloomFilterHeaderAAD);
         } catch (IOException e) {
             LOG.warn("read no bloom filter");
             return null;
@@ -987,9 +1051,9 @@ public class ParquetFileReader implements Closeable {
         byte[] bitset;
         if (null == bloomFilterDecryptor) {
             bitset = new byte[numBytes];
-            f.readFully(bitset);
+            in.read(bitset);
         } else {
-            bitset = bloomFilterDecryptor.decrypt(f, bloomFilterBitsetAAD);
+            bitset = bloomFilterDecryptor.decrypt(in, bloomFilterBitsetAAD);
             if (bitset.length != numBytes) {
                 throw new ParquetCryptoRuntimeException(
                         "Wrong length of decrypted bloom filter bitset");
@@ -999,11 +1063,12 @@ public class ParquetFileReader implements Closeable {
     }
 
     /**
+     * Class should be considered private.
+     *
      * @param column the column chunk which the column index is to be returned for
      * @return the column index for the specified column chunk or {@code null} if there is no index
      * @throws IOException if any I/O error occurs during reading the file
      */
-    @Private
     public ColumnIndex readColumnIndex(ColumnChunkMetaData column) throws IOException {
         IndexReference ref = column.getColumnIndexReference();
         if (ref == null) {
@@ -1033,11 +1098,12 @@ public class ParquetFileReader implements Closeable {
     }
 
     /**
+     * Class should be considered private.
+     *
      * @param column the column chunk which the offset index is to be returned for
      * @return the offset index for the specified column chunk or {@code null} if there is no index
      * @throws IOException if any I/O error occurs during reading the file
      */
-    @Private
     public OffsetIndex readOffsetIndex(ColumnChunkMetaData column) throws IOException {
         IndexReference ref = column.getOffsetIndexReference();
         if (ref == null) {
@@ -1072,6 +1138,7 @@ public class ParquetFileReader implements Closeable {
                 f.close();
             }
         } finally {
+            AutoCloseables.uncheckedClose(nextDictionaryReader, crcAllocator);
             options.getCodecFactory().release();
         }
     }
@@ -1090,6 +1157,7 @@ public class ParquetFileReader implements Closeable {
         private ChunkDescriptor lastDescriptor;
         private final long rowCount;
         private SeekableInputStream f;
+        private final ByteBufferReleaser releaser = new ByteBufferReleaser(options.getAllocator());
 
         public ChunkListBuilder(long rowCount) {
             this.rowCount = rowCount;
@@ -1101,6 +1169,10 @@ public class ParquetFileReader implements Closeable {
             this.f = f;
         }
 
+        void addBuffersToRelease(List<ByteBuffer> toRelease) {
+            toRelease.forEach(releaser::releaseLater);
+        }
+
         void setOffsetIndex(ChunkDescriptor descriptor, OffsetIndex offsetIndex) {
             map.computeIfAbsent(descriptor, d -> new ChunkData()).offsetIndex = offsetIndex;
         }
@@ -1161,9 +1233,11 @@ public class ParquetFileReader implements Closeable {
          * Calculate checksum of input bytes, throw decoding exception if it does not match the
          * provided reference crc.
          */
-        private void verifyCrc(int referenceCrc, byte[] bytes, String exceptionMsg) {
+        private void verifyCrc(int referenceCrc, BytesInput bytes, String exceptionMsg) {
             crc.reset();
-            crc.update(bytes);
+            try (ByteBufferReleaser releaser = crcAllocator.getReleaser()) {
+                crc.update(bytes.toByteBuffer(releaser));
+            }
             if (crc.getValue() != ((long) referenceCrc & 0xffffffffL)) {
                 throw new ParquetDecodingException(exceptionMsg);
             }
@@ -1237,7 +1311,7 @@ public class ParquetFileReader implements Closeable {
                         if (options.usePageChecksumVerification() && pageHeader.isSetCrc()) {
                             verifyCrc(
                                     pageHeader.getCrc(),
-                                    pageBytes.toByteArray(),
+                                    pageBytes,
                                     "could not verify dictionary page integrity, CRC checksum verification failed");
                         }
                         DictionaryPageHeader dicHeader = pageHeader.getDictionary_page_header();
@@ -1258,7 +1332,7 @@ public class ParquetFileReader implements Closeable {
                         if (options.usePageChecksumVerification() && pageHeader.isSetCrc()) {
                             verifyCrc(
                                     pageHeader.getCrc(),
-                                    pageBytes.toByteArray(),
+                                    pageBytes,
                                     "could not verify page integrity, CRC checksum verification failed");
                         }
                         DataPageV1 dataPageV1 =
@@ -1289,23 +1363,41 @@ public class ParquetFileReader implements Closeable {
                                 compressedPageSize
                                         - dataHeaderV2.getRepetition_levels_byte_length()
                                         - dataHeaderV2.getDefinition_levels_byte_length();
-                        pagesInChunk.add(
+                        final BytesInput repetitionLevels =
+                                this.readAsBytesInput(
+                                        dataHeaderV2.getRepetition_levels_byte_length());
+                        final BytesInput definitionLevels =
+                                this.readAsBytesInput(
+                                        dataHeaderV2.getDefinition_levels_byte_length());
+                        final BytesInput values = this.readAsBytesInput(dataSize);
+                        if (options.usePageChecksumVerification() && pageHeader.isSetCrc()) {
+                            pageBytes =
+                                    BytesInput.concat(repetitionLevels, definitionLevels, values);
+                            verifyCrc(
+                                    pageHeader.getCrc(),
+                                    pageBytes,
+                                    "could not verify page integrity, CRC checksum verification failed");
+                        }
+                        DataPageV2 dataPageV2 =
                                 new DataPageV2(
                                         dataHeaderV2.getNum_rows(),
                                         dataHeaderV2.getNum_nulls(),
                                         dataHeaderV2.getNum_values(),
-                                        this.readAsBytesInput(
-                                                dataHeaderV2.getRepetition_levels_byte_length()),
-                                        this.readAsBytesInput(
-                                                dataHeaderV2.getDefinition_levels_byte_length()),
+                                        repetitionLevels,
+                                        definitionLevels,
                                         converter.getEncoding(dataHeaderV2.getEncoding()),
-                                        this.readAsBytesInput(dataSize),
+                                        values,
                                         uncompressedPageSize,
                                         converter.fromParquetStatistics(
                                                 getFileMetaData().getCreatedBy(),
                                                 dataHeaderV2.getStatistics(),
                                                 type),
-                                        dataHeaderV2.isIs_compressed()));
+                                        dataHeaderV2.isIs_compressed());
+                        // Copy crc to new page, used for testing
+                        if (pageHeader.isSetCrc()) {
+                            dataPageV2.setCrc(pageHeader.getCrc());
+                        }
+                        pagesInChunk.add(dataPageV2);
                         valuesCountReadSoFar += dataHeaderV2.getNum_values();
                         ++dataPageCountReadSoFar;
                         break;
@@ -1346,7 +1438,8 @@ public class ParquetFileReader implements Closeable {
                     pageBlockDecryptor,
                     aadPrefix,
                     rowGroupOrdinal,
-                    columnOrdinal);
+                    columnOrdinal,
+                    options);
         }
 
         private boolean hasMorePages(long valuesCountReadSoFar, int dataPageCountReadSoFar) {
@@ -1528,11 +1621,14 @@ public class ParquetFileReader implements Closeable {
             if (lastAllocationSize > 0) {
                 buffers.add(options.getAllocator().allocate(lastAllocationSize));
             }
+            builder.addBuffersToRelease(buffers);
 
+            long readStart = System.nanoTime();
             for (ByteBuffer buffer : buffers) {
                 f.readFully(buffer);
                 buffer.flip();
             }
+            setReadMetrics(readStart, length);
 
             // report in a counter the data we just scanned
             BenchmarkCounter.incrementBytesRead(length);
@@ -1542,24 +1638,60 @@ public class ParquetFileReader implements Closeable {
             }
         }
 
+        private void setReadMetrics(long startNs, long len) {
+            ParquetMetricsCallback metricsCallback = options.getMetricsCallback();
+            if (metricsCallback != null) {
+                long totalFileReadTimeNs = Math.max(System.nanoTime() - startNs, 0);
+                double sizeInMb = ((double) len) / (1024 * 1024);
+                double timeInSec = ((double) totalFileReadTimeNs) / 1000_0000_0000L;
+                double throughput = sizeInMb / timeInSec;
+                LOG.debug(
+                        "Parquet: File Read stats:  Length: {} MB, Time: {} secs, throughput: {} MB/sec ",
+                        sizeInMb,
+                        timeInSec,
+                        throughput);
+                metricsCallback.setDuration(
+                        ParquetFileReaderMetrics.ReadTime.name(), totalFileReadTimeNs);
+                metricsCallback.setValueLong(ParquetFileReaderMetrics.ReadSize.name(), length);
+                metricsCallback.setValueDouble(
+                        ParquetFileReaderMetrics.ReadThroughput.name(), throughput);
+            }
+        }
+
         /**
-         * Populate data in a parquet file range from a vectored range.
+         * Populate data in a parquet file range from a vectored range; will block for up to {@link
+         * #HADOOP_VECTORED_READ_TIMEOUT_SECONDS} seconds.
          *
          * @param currRange range to populated.
          * @param builder used to build chunk list to read the pages for the different columns.
          * @throws IOException if there is an error while reading from the stream, including a
          *     timeout.
          */
-        public void readFromVectoredRange(FileRange currRange, ChunkListBuilder builder)
+        public void readFromVectoredRange(ParquetFileRange currRange, ChunkListBuilder builder)
                 throws IOException {
-            byte[] buffer;
+            ByteBuffer buffer;
+            final long timeoutSeconds = HADOOP_VECTORED_READ_TIMEOUT_SECONDS;
+            long readStart = System.nanoTime();
             try {
-                buffer = currRange.getData().get();
-            } catch (InterruptedException | ExecutionException e) {
-                throw new RuntimeException(e);
+                LOG.debug(
+                        "Waiting for vectored read to finish for range {} with timeout {} seconds",
+                        currRange,
+                        timeoutSeconds);
+                buffer =
+                        FutureIO.awaitFuture(
+                                currRange.getDataReadFuture(), timeoutSeconds, TimeUnit.SECONDS);
+                setReadMetrics(readStart, currRange.getLength());
+                // report in a counter the data we just scanned
+                BenchmarkCounter.incrementBytesRead(currRange.getLength());
+            } catch (TimeoutException e) {
+                String error =
+                        String.format(
+                                "Timeout while fetching result for %s with time limit %d seconds",
+                                currRange, timeoutSeconds);
+                LOG.error(error, e);
+                throw new IOException(error, e);
             }
-
-            ByteBufferInputStream stream = ByteBufferInputStream.wrap(ByteBuffer.wrap(buffer));
+            ByteBufferInputStream stream = ByteBufferInputStream.wrap(buffer);
             for (ChunkDescriptor descriptor : chunks) {
                 builder.add(descriptor, stream.sliceBuffers(descriptor.size), f);
             }
