@@ -19,6 +19,7 @@
 package org.apache.paimon.spark
 
 import org.apache.paimon.predicate.Predicate
+import org.apache.paimon.spark.commands.BucketExpression.quote
 import org.apache.paimon.table.{BucketMode, FileStoreTable, Table}
 import org.apache.paimon.table.source.{DataSplit, Split}
 
@@ -62,11 +63,16 @@ case class PaimonScan(
           // so we only support one bucket key case.
           assert(bucketSpec.getNumBuckets > 0)
           assert(bucketSpec.getBucketKeys.size() == 1)
-          val bucketKey = bucketSpec.getBucketKeys.get(0)
-          if (requiredSchema.exists(f => conf.resolver(f.name, bucketKey))) {
-            Some(Expressions.bucket(bucketSpec.getNumBuckets, bucketKey))
-          } else {
-            None
+          extractBucketNumber() match {
+            case Some(num) =>
+              val bucketKey = bucketSpec.getBucketKeys.get(0)
+              if (requiredSchema.exists(f => conf.resolver(f.name, bucketKey))) {
+                Some(Expressions.bucket(num, quote(bucketKey)))
+              } else {
+                None
+              }
+
+            case _ => None
           }
         }
 
@@ -74,6 +80,24 @@ case class PaimonScan(
     }
   }
 
+  /**
+   * Extract the bucket number from the splits only if all splits have the same totalBuckets number.
+   */
+  private def extractBucketNumber(): Option[Int] = {
+    val splits = getOriginSplits
+    if (splits.exists(!_.isInstanceOf[DataSplit])) {
+      None
+    } else {
+      val deduplicated =
+        splits.map(s => Option(s.asInstanceOf[DataSplit].totalBuckets())).toSeq.distinct
+
+      deduplicated match {
+        case Seq(Some(num)) => Some(num)
+        case _ => None
+      }
+    }
+  }
+
   private def shouldDoBucketedScan: Boolean = {
     !bucketedScanDisabled && conf.v2BucketingEnabled && extractBucketTransform.isDefined
   }
@@ -169,6 +193,7 @@ case class PaimonScan(
       readBuilder.withFilter(partitionFilter.toList.asJava)
       // set inputPartitions null to trigger to get the new splits.
       inputPartitions = null
+      inputSplits = null
     }
   }
 }
