@@ -33,7 +33,6 @@ import org.apache.spark.sql.catalyst.expressions.Literal.TrueLiteral
 import org.apache.spark.sql.catalyst.plans.logical.{Assignment, Filter, Project, SupportsSubquery}
 import org.apache.spark.sql.execution.datasources.v2.DataSourceV2Relation
 import org.apache.spark.sql.functions.lit
-import org.apache.spark.sql.paimon.shims.SparkShimLoader
 
 case class UpdatePaimonTableCommand(
     relation: DataSourceV2Relation,
@@ -96,7 +95,7 @@ case class UpdatePaimonTableCommand(
         try {
           // Step3: write these updated data
           val touchedDataSplits = deletionVectors.collect().map {
-            SparkDeletionVectors.toDataSplit(_, root, pathFactory, dataFilePathToMeta)
+            SparkDeletionVector.toDataSplit(_, root, pathFactory, dataFilePathToMeta)
           }
           val addCommitMessage = writeOnlyUpdatedData(sparkSession, touchedDataSplits)
 
@@ -132,7 +131,7 @@ case class UpdatePaimonTableCommand(
       touchedDataSplits: Array[DataSplit]): Seq[CommitMessage] = {
     val updateColumns = updateExpressions.zip(relation.output).map {
       case (update, origin) =>
-        SparkShimLoader.getSparkShim.column(update).as(origin.name, origin.metadata)
+        toColumn(update).as(origin.name, origin.metadata)
     }
 
     val toUpdateScanRelation = createNewRelation(touchedDataSplits, relation)
@@ -155,7 +154,7 @@ case class UpdatePaimonTableCommand(
         } else {
           If(condition, update, origin)
         }
-        SparkShimLoader.getSparkShim.column(updated).as(origin.name, origin.metadata)
+        toColumn(updated).as(origin.name, origin.metadata)
     }
 
     val data = createDataset(sparkSession, toUpdateScanRelation).select(updateColumns: _*)
