@@ -21,6 +21,7 @@ package org.apache.paimon.spark.util
 import org.apache.paimon.types.RowKind
 
 import org.apache.spark.sql.Row
+import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.types.StructType
 
 object SparkRowUtils {
@@ -33,6 +34,14 @@ object SparkRowUtils {
     }
   }
 
+  def getRowKind(row: InternalRow, rowkindColIdx: Int): RowKind = {
+    if (rowkindColIdx != -1) {
+      RowKind.fromByteValue(row.getByte(rowkindColIdx))
+    } else {
+      RowKind.INSERT
+    }
+  }
+
   def getFieldIndex(schema: StructType, colName: String): Int = {
     try {
       schema.fieldIndex(colName)
