@@ -43,6 +43,7 @@ import java.util.List;
 import java.util.stream.Collectors;
 
 import static org.assertj.core.api.Assertions.assertThat;
+import static org.assertj.core.api.Assertions.assertThatThrownBy;
 
 /** ITCase for spark writer. */
 @TestInstance(TestInstance.Lifecycle.PER_CLASS)
@@ -78,7 +79,49 @@ public class SparkWriteITCase {
 
     @AfterEach
     public void afterEach() {
-        spark.sql("DROP TABLE T");
+        spark.sql("DROP TABLE IF EXISTS T");
+    }
+
+    @Test
+    public void testWriteWithDefaultValue() {
+        spark.sql(
+                "CREATE TABLE T (a INT, b INT DEFAULT 2, c STRING DEFAULT 'my_value') TBLPROPERTIES"
+                        + " ('file.format'='avro')");
+
+        // test show create table
+        List<Row> show = spark.sql("SHOW CREATE TABLE T").collectAsList();
+        assertThat(show.toString())
+                .contains("a INT,\n" + "  b INT DEFAULT 2,\n" + "  c STRING DEFAULT 'my_value'");
+
+        // test partial write
+        spark.sql("INSERT INTO T (a) VALUES (1), (2)").collectAsList();
+        List<Row> rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString()).isEqualTo("[[1,2,my_value], [2,2,my_value]]");
+
+        // test write with DEFAULT
+        spark.sql("INSERT INTO T VALUES (3, DEFAULT, DEFAULT)").collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString()).isEqualTo("[[1,2,my_value], [2,2,my_value], [3,2,my_value]]");
+
+        // test add column with DEFAULT not support
+        assertThatThrownBy(() -> spark.sql("ALTER TABLE T ADD COLUMN d INT DEFAULT 5"))
+                .hasMessageContaining(
+                        "Unsupported table change: Cannot add column [d] with default value");
+
+        // test alter type to default column
+        spark.sql("ALTER TABLE T ALTER COLUMN b TYPE STRING").collectAsList();
+        spark.sql("INSERT INTO T (a) VALUES (4)").collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString())
+                .isEqualTo("[[1,2,my_value], [2,2,my_value], [3,2,my_value], [4,2,my_value]]");
+
+        // test alter default column
+        spark.sql("ALTER TABLE T ALTER COLUMN b SET DEFAULT '3'");
+        spark.sql("INSERT INTO T (a) VALUES (5)").collectAsList();
+        rows = spark.sql("SELECT * FROM T").collectAsList();
+        assertThat(rows.toString())
+                .isEqualTo(
+                        "[[1,2,my_value], [2,2,my_value], [3,2,my_value], [4,2,my_value], [5,3,my_value]]");
     }
 
     @Test
