@@ -101,39 +101,6 @@ abstract class DDLTestBase extends PaimonSparkTestBase {
     }
   }
 
-  test("Paimon DDL: Create Table As Select") {
-    withTable("source", "t1", "t2") {
-      Seq((1L, "x1", "2023"), (2L, "x2", "2023"))
-        .toDF("a", "b", "pt")
-        .createOrReplaceTempView("source")
-
-      spark.sql("""
-                  |CREATE TABLE t1 AS SELECT * FROM source
-                  |""".stripMargin)
-      val t1 = loadTable("t1")
-      Assertions.assertTrue(t1.primaryKeys().isEmpty)
-      Assertions.assertTrue(t1.partitionKeys().isEmpty)
-
-      spark.sql(
-        """
-          |CREATE TABLE t2
-          |PARTITIONED BY (pt)
-          |TBLPROPERTIES ('bucket' = '5', 'primary-key' = 'a,pt', 'target-file-size' = '128MB')
-          |AS SELECT * FROM source
-          |""".stripMargin)
-      val t2 = loadTable("t2")
-      Assertions.assertEquals(2, t2.primaryKeys().size())
-      Assertions.assertTrue(t2.primaryKeys().contains("a"))
-      Assertions.assertTrue(t2.primaryKeys().contains("pt"))
-      Assertions.assertEquals(1, t2.partitionKeys().size())
-      Assertions.assertEquals("pt", t2.partitionKeys().get(0))
-
-      // check all the core options
-      Assertions.assertEquals("5", t2.options().get("bucket"))
-      Assertions.assertEquals("128MB", t2.options().get("target-file-size"))
-    }
-  }
-
   test("Paimon DDL: create database with location with filesystem catalog") {
     withTempDir {
       dBLocation =>
